import pytest
from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse
from mock import Mock


def test_Uczelnia_wosclient(uczelnia):
    with pytest.raises(ImproperlyConfigured):
        uczelnia.wosclient()

    uczelnia.clarivate_username = "foo"
    uczelnia.clarivate_password = "bar"
    res = uczelnia.wosclient()
    assert res is not None


def test_GetWoSAMRInformation_post_no_args(wd_app, uczelnia):
    res = wd_app.post(reverse("bpp:api_wos_amr", args=(uczelnia.slug,)))
    assert res.json["status"] == "error"


def test_GetWoSAMRInformation_post_good(wd_app, uczelnia, mocker):
    m = Mock()
    m.query_single = Mock(return_value={"timesCited": "-1"})
    mocker.patch("bpp.models.struktura.Uczelnia.wosclient", return_value=m)

    res = wd_app.post(
        reverse("bpp:api_wos_amr", args=(uczelnia.slug,)), params={"pmid": "31337"}
    )

    assert res.json["status"] == "ok"
    assert res.json["timesCited"] == "-1"


def test_GetWoSAMRInformation_post_error(wd_app, uczelnia, mocker):
    m = Mock()
    m.query_single = Mock(side_effect=Exception("lel"))
    mocker.patch("bpp.models.struktura.Uczelnia.wosclient", return_value=m)

    res = wd_app.post(
        reverse("bpp:api_wos_amr", args=(uczelnia.slug,)), params={"pmid": "31337"}
    )

    assert res.json["status"] == "error"
    assert res.json["info"].find("lel") >= 0



from django import forms

from miniblog.admin import SmallerTextarea
from ..models.konferencja import Konferencja
from .core import BaseBppAdminMixin
from .helpers import ADNOTACJE_FIELDSET

from django.contrib import admin

from bpp.admin.helpers import LimitingFormset
from bpp.models import Wydawnictwo_Ciagle, Wydawnictwo_Zwarte


class Wydawnictwo_Zwarte_Konferencja_Form(forms.ModelForm):
    model = Wydawnictwo_Zwarte

    class Meta:
        fields = [
            "tytul_oryginalny",
            "charakter_formalny",
            "typ_kbn",
            "rok",
            "jezyk",
            "status_korekty",
        ]
        widgets = {"tytul_oryginalny": SmallerTextarea}


class Wydawnictwo_Ciagle_Konferencja_Form(forms.ModelForm):
    model = Wydawnictwo_Ciagle

    class Meta:
        fields = [
            "tytul_oryginalny",
            "charakter_formalny",
            "typ_kbn",
            "rok",
            "jezyk",
            "status_korekty",
        ]
        widgets = {"tytul_oryginalny": SmallerTextarea}


class Wydawnictwo_Zwarte_KonferencjaInline(admin.TabularInline):
    form = Wydawnictwo_Zwarte_Konferencja_Form
    model = Wydawnictwo_Zwarte
    formset = LimitingFormset
    extra = 0


class Wydawnictwo_Ciagle_KonferencjaInline(admin.TabularInline):
    form = Wydawnictwo_Ciagle_Konferencja_Form
    model = Wydawnictwo_Ciagle
    formset = LimitingFormset
    extra = 0


class KonferencjaAdmin(BaseBppAdminMixin, admin.ModelAdmin):
    list_display = [
        "nazwa",
        "typ_konferencji",
        "rozpoczecie",
        "zakonczenie",
        "miasto",
        "panstwo",
        "baza_scopus",
        "baza_wos",
        "pbn_uid",
    ]
    list_filter = [
        "miasto",
        "panstwo",
        "rozpoczecie",
        "zakonczenie",
        "baza_scopus",
        "baza_wos",
        "baza_inna",
        "typ_konferencji",
    ]
    search_fields = [
        "nazwa",
        "rozpoczecie",
        "zakonczenie",
        "miasto",
        "panstwo",
        "pbn_uid__pk",
    ]
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "nazwa",
                    "skrocona_nazwa",
                    "typ_konferencji",
                    "rozpoczecie",
                    "zakonczenie",
                    "miasto",
                    "panstwo",
                    "baza_scopus",
                    "baza_wos",
                    "baza_inna",
                    "pbn_uid",
                )
            },
        ),
        ADNOTACJE_FIELDSET,
    )
    inlines = [
        Wydawnictwo_Zwarte_KonferencjaInline,
        Wydawnictwo_Ciagle_KonferencjaInline,
    ]

    readonly_fields = ["ostatnio_zmieniony"]

    autocomplete_fields = [
        "pbn_uid",
    ]


admin.site.register(Konferencja, KonferencjaAdmin)



import pytest
from model_bakery import baker

from bpp.models import (
    Autor,
    Jednostka,
    Typ_Odpowiedzialnosci,
    Tytul,
    Wydawnictwo_Ciagle,
)


@pytest.mark.django_db
def test_Autor_str(tytuly):
    x = Autor(
        nazwisko="Kowalski",
        imiona="Jan",
        tytul=Tytul.objects.first(),
        poprzednie_nazwiska="Budnik",
        pseudonim="Fafa",
    )
    assert str(x) == "Kowalski Jan (Budnik), dr (Fafa)"


@pytest.mark.django_db
def test_Autor_liczba_cytowan():
    Typ_Odpowiedzialnosci.objects.get_or_create(skrot="aut.", nazwa="autor")
    autor = baker.make(Autor)
    jednostka = baker.make(Jednostka, skupia_pracownikow=True)
    wc = baker.make(Wydawnictwo_Ciagle, liczba_cytowan=200)
    wc.dodaj_autora(autor, jednostka, zapisany_jako="Jan K")

    j2 = baker.make(Jednostka, skupia_pracownikow=False)
    wc2 = baker.make(Wydawnictwo_Ciagle, liczba_cytowan=300)
    wc2.dodaj_autora(autor, j2, zapisany_jako="Jan K2", afiliuje=False)

    assert autor.liczba_cytowan() == 500


@pytest.mark.django_db
def test_liczba_cytowan_afiliowane():
    Typ_Odpowiedzialnosci.objects.get_or_create(skrot="aut.", nazwa="autor")
    autor = baker.make(Autor)
    jednostka = baker.make(Jednostka, skupia_pracownikow=True)
    wc = baker.make(Wydawnictwo_Ciagle, liczba_cytowan=200)
    wc.dodaj_autora(autor, jednostka, zapisany_jako="Jan K")

    j2 = baker.make(Jednostka, skupia_pracownikow=False)
    wc2 = baker.make(Wydawnictwo_Ciagle, liczba_cytowan=300)
    wc2.dodaj_autora(autor, j2, zapisany_jako="Jan K2", afiliuje=False)

    assert autor.liczba_cytowan_afiliowane() == 200



from datetime import date

import pytest

from bpp.tests import normalize_html

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from model_bakery import baker

from fixtures import NORMAL_DJANGO_USER_LOGIN, NORMAL_DJANGO_USER_PASSWORD

from bpp.models import (
    Autor_Jednostka,
    Funkcja_Autora,
    Jednostka,
    OpcjaWyswietlaniaField,
    Praca_Doktorska,
    Typ_Odpowiedzialnosci,
    Wydawnictwo_Ciagle,
)
from bpp.models.autor import Autor


def test_autor_ukrywanie_nazwisk(autor_jan_nowak, client, admin_client):
    NAZWISKO = "NazwiskoAutora"
    autor_jan_nowak.poprzednie_nazwiska = NAZWISKO
    assert NAZWISKO in str(autor_jan_nowak)

    autor_jan_nowak.pokazuj_poprzednie_nazwiska = False
    autor_jan_nowak.save()

    assert NAZWISKO not in str(autor_jan_nowak)

    url = reverse("bpp:browse_autor", args=(autor_jan_nowak.slug,))

    page = admin_client.get(url)
    assert NAZWISKO in normalize_html(page.rendered_content)

    page = client.get(url)
    assert NAZWISKO not in normalize_html(page.rendered_content)


@pytest.mark.django_db
def test_AutorView_funkcja_za_nazwiskiem(app):
    autor = baker.make(Autor, nazwisko="Foo", imiona="Bar")
    jednostka = baker.make(Jednostka, nazwa="Jednostka")
    funkcja = Funkcja_Autora.objects.create(
        nazwa="profesor uczelni", skrot="prof. ucz."
    )
    aj = Autor_Jednostka.objects.create(
        autor=autor, jednostka=jednostka, funkcja=funkcja
    )

    url = reverse("bpp:browse_autor", args=(autor.slug,))

    page = app.get(url)
    assert page.status_code == 200
    res = normalize_html(str(page.content, "utf-8"))
    assert res.find("<h1>Foo Bar </h1>") >= 0

    aj.rozpoczal_prace = date(2020, 1, 1)
    aj.save()

    page = app.get(url)
    res = normalize_html(str(page.content, "utf-8"))
    assert res.find("<h1>Foo Bar </h1>") >= 0

    funkcja.pokazuj_za_nazwiskiem = True
    funkcja.save()

    page = app.get(url)
    res = normalize_html(str(page.content, "utf-8"))
    assert res.find("<h1>Foo Bar, profesor uczelni </h1>") >= 0


@pytest.fixture
def test_browse_autor():
    Typ_Odpowiedzialnosci.objects.get_or_create(nazwa="autor", skrot="aut.")

    autor = baker.make(Autor)
    jednostka = baker.make(Jednostka, skupia_pracownikow=True)
    wc = baker.make(Wydawnictwo_Ciagle, liczba_cytowan=200)
    wc.dodaj_autora(autor, jednostka, zapisany_jako="Jan K")

    j2 = baker.make(Jednostka, skupia_pracownikow=False)
    wc2 = baker.make(Wydawnictwo_Ciagle, liczba_cytowan=300)
    wc2.dodaj_autora(autor, j2, zapisany_jako="Jan K2", afiliuje=False)

    return autor


def test_browse_autor_dwa_doktoraty(typy_odpowiedzialnosci, autor_jan_kowalski, client):
    tytuly_prac = ["Praca 1", "Praca 2"]
    for praca in tytuly_prac:
        baker.make(Praca_Doktorska, tytul_oryginalny=praca, autor=autor_jan_kowalski)

    res = client.get(
        reverse(
            "bpp:browse_autor",
            kwargs=dict(
                slug=autor_jan_kowalski.slug,
            ),
        )
    )

    for praca in tytuly_prac:
        assert praca in res.content.decode("utf-8")


@pytest.mark.django_db
def test_browse_autor_podstrona_liczba_cytowan_nigdy(
    client, uczelnia, test_browse_autor
):
    uczelnia.pokazuj_liczbe_cytowan_na_stronie_autora = (
        OpcjaWyswietlaniaField.POKAZUJ_NIGDY
    )
    uczelnia.save()

    res = client.get(reverse("bpp:browse_autor", args=(test_browse_autor.slug,)))
    assert "Liczba cytowań" not in res.rendered_content


@pytest.mark.django_db
def test_browse_autor_podstrona_liczba_cytowan_zawsze(
    client, uczelnia, test_browse_autor
):
    uczelnia.pokazuj_liczbe_cytowan_na_stronie_autora = (
        OpcjaWyswietlaniaField.POKAZUJ_ZAWSZE
    )
    uczelnia.save()

    res = client.get(reverse("bpp:browse_autor", args=(test_browse_autor.slug,)))

    content = normalize_html(res.rendered_content)
    assert "Liczba cytowań" in content
    assert "Liczba cytowań: </strong>500" in content
    assert "Liczba cytowań z jednostek afiliowanych: </strong>200" in content


@pytest.mark.django_db
def test_browse_autor_podstrona_liczba_cytowan_zalogowani(
    client, uczelnia, test_browse_autor, normal_django_user
):
    uczelnia.pokazuj_liczbe_cytowan_na_stronie_autora = (
        OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
    )
    uczelnia.save()

    res = client.get(reverse("bpp:browse_autor", args=(test_browse_autor.slug,)))
    assert "Liczba cytowań" not in res.rendered_content

    client.login(
        username=NORMAL_DJANGO_USER_LOGIN, password=NORMAL_DJANGO_USER_PASSWORD
    )
    res = client.get(reverse("bpp:browse_autor", args=(test_browse_autor.slug,)))
    assert "Liczba cytowań" in res.rendered_content



from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.contenttypes.models import ContentType
from django.db import transaction
from django.http import Http404, HttpResponseRedirect
from django.template.response import TemplateResponse
from django.utils.functional import cached_property
from django.views.generic import CreateView, DetailView, ListView

from long_running import const
from long_running.tasks import perform_generic_long_running_task


class LongRunningTaskCallerMixin:
    task = perform_generic_long_running_task

    def task_on_commit(self, pk):
        ct = ContentType.objects.get_for_model(self.model)

        transaction.on_commit(lambda: self.task.delay(ct.app_label, ct.model, pk))


class RestrictToOwnerMixin(LoginRequiredMixin):
    def get_queryset(self):
        return self.model.objects.filter(owner=self.request.user)


class LongRunningOperationsView(RestrictToOwnerMixin, ListView):
    max_previous_ops = 10

    def get_queryset(self):
        qset = RestrictToOwnerMixin.get_queryset(self).order_by("-last_updated_on")

        # Skasju poprzednie operacje
        for elem in qset[self.max_previous_ops :]:
            elem.delete()

        return qset


class LongRunningSingleObjectChannelSubscriberMixin:
    def get_context_data(self, **kwargs):
        return super(
            LongRunningSingleObjectChannelSubscriberMixin, self
        ).get_context_data(extraChannels=[self.object.pk], **kwargs)


class LongRunningRouterView(
    RestrictToOwnerMixin, LongRunningSingleObjectChannelSubscriberMixin, DetailView
):
    """You can mount this view somewhere on an url with a pk (an uuid) of
    an Operation and it will try routing to either a web page with the progress
    report, or in case operation finishes, to a result page (or an error page...)"""

    redirect_prefix = None

    STATE_TO_SUFFIX_MAP = {
        const.PROCESSING_STARTED: "details",
        const.PROCESSING_FINISHED_WITH_ERROR: "details",
        const.PROCESSING_FINISHED_SUCCESSFULLY: "results",
    }

    def get(self, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data()
        state = self.object.get_state()

        if state == const.PROCESSING_NOT_STARTED:
            # If not started, redirect to itself but after a few seconds
            return TemplateResponse(
                request=self.request,
                template="long_running/router_view_wait.html",
                context=context,
            )

        suffix = self.STATE_TO_SUFFIX_MAP.get(state)

        url = self.object.get_url(suffix)

        return HttpResponseRedirect(url)


class LongRunningDetailsView(
    RestrictToOwnerMixin, LongRunningSingleObjectChannelSubscriberMixin, DetailView
):
    pass


class LongRunningResultsView(ListView):
    paginate_by = 25

    @cached_property
    def parent_object(self):
        o = self.model.objects.get(pk=self.kwargs["pk"])
        if o.owner != self.request.user:
            raise Http404
        return o

    def get_queryset(self):
        return self.parent_object.get_details_set()

    def get_context_data(self, **kwargs):
        return super(LongRunningResultsView, self).get_context_data(
            extraChannels=[self.parent_object.pk], object=self.parent_object, **kwargs
        )


class CreateLongRunningOperationView(
    LoginRequiredMixin, LongRunningTaskCallerMixin, CreateView
):
    def form_valid(self, form):
        form.instance.owner = self.request.user
        self.object = form.save()
        self.task_on_commit(pk=form.instance.pk)
        return HttpResponseRedirect(self.get_success_url())


class RestartLongRunningOperationView(
    RestrictToOwnerMixin, LongRunningTaskCallerMixin, DetailView
):
    # task = perform_generic_long_running_task

    @transaction.atomic
    def get(self, *args, **kw):
        self.object = self.get_object()
        if self.object.finished_on is not None:
            self.object.mark_reset()
            self.task_on_commit(pk=self.object.pk)
        return HttpResponseRedirect("..")



from datetime import date


def test_odepnij_autorow_spoza_pliku_autor_jest_w_pliku_ale_odpinamy_inne_miejsca_zarzadzaj_automatycznie(
    import_pracownikow_performed,
    autor_z_pliku,
    jednostka_z_pliku,
    jednostka_spoza_pliku,
    today,
    yesterday,
):
    aj_spoza_pliku = autor_z_pliku.dodaj_jednostke(jednostka_spoza_pliku)
    jednostka_spoza_pliku.zarzadzaj_automatycznie = True
    jednostka_spoza_pliku.save()

    # Odpinamy
    import_pracownikow_performed.odepnij_autorow_spoza_pliku(
        today=today, yesterday=yesterday
    )

    # Odpięcie automatyczne jednostek zarządzanych automatycznie powinno odpiąc te jednostki
    # a w sytuacji, gdy autorowi nie pozostanie zadna jednostka, aktualna jednostka
    # będzie miała wartość None:

    autor_z_pliku.refresh_from_db()
    assert autor_z_pliku.aktualna_jednostka == jednostka_z_pliku

    # Przypisanie Autor+Jednostka znajdujące się w pliku zostanie dodane od dzisiaj
    assert autor_z_pliku.autor_jednostka_set.get(
        jednostka=jednostka_z_pliku
    ).rozpoczal_prace == date(2016, 10, 1)

    # Przypisania Autor + Jednostka spoza pliku bedzie miała "odpięte" miejsce pracy
    # czyli datę zakończenia równą wczoraj
    aj_spoza_pliku.refresh_from_db()
    assert aj_spoza_pliku.zakonczyl_prace == yesterday
    assert aj_spoza_pliku.podstawowe_miejsce_pracy is False


def test_odepnij_autorow_spoza_pliku_autor_jest_w_pliku_ale_odpinamy_inne_miejsca_nie_zarzadzaj_automatycznie(
    import_pracownikow_performed,
    autor_z_pliku,
    jednostka_z_pliku,
    jednostka_spoza_pliku,
    today,
    yesterday,
):
    autor_z_pliku.dodaj_jednostke(jednostka_spoza_pliku)
    jednostka_spoza_pliku.zarzadzaj_automatycznie = False
    jednostka_spoza_pliku.save()

    # Zweryfikujmy stan po imporcie -- `autor_z_pliku` przypięty do `jednostka_z_pliku`
    autor_z_pliku.refresh_from_db()
    assert autor_z_pliku.aktualna_jednostka == jednostka_z_pliku

    # Przy wykorzystaniu przez tą funkcję parametru 'import_pracownikow_performed', mamy w bazie
    # danych stan taki, jak w pliku -- czyli "podstawowe miejsce pracy" ustawione dla pana
    # `autor_z_pliku` na `jednostka_z_pliku`. Z tym, że teraz chcemy przetestować odpinanie jednostek
    # zatem skasujemy tą informację z pliku importu, aby było tak jak gdyby tego wpisu w pliku
    # importu nie było. Zatem, oczekiwanym zachowaniem będzie przez system odpięcie jednostki
    # `jednostka_z_pliku` (bo jest zarządzana automatycznie) oraz NIE odpięcie jednostki `jednostka_spoza_pliku`
    # gdyż ona ma `zarzadzaj_automatycznie` ustawione na False.

    import_pracownikow_performed.importpracownikowrow_set.all().delete()

    # Skasowana.

    # Teraz odpinamy:
    import_pracownikow_performed.odepnij_autorow_spoza_pliku(
        today=today, yesterday=yesterday
    )

    # Odpięcie jednostek nie-zarządzanych automatycznie powinno ich nie ruszać:

    autor_z_pliku.refresh_from_db()
    assert autor_z_pliku.aktualna_jednostka == jednostka_spoza_pliku


def test_odepnij_autorow_spoza_pliku_inny_autor_czy_odepnie_automatyczna(
    import_pracownikow_performed,
    autor_z_pliku,
    jednostka_z_pliku,
    autor_spoza_pliku,
    jednostka_spoza_pliku,
):
    autor_spoza_pliku.dodaj_jednostke(jednostka_spoza_pliku)
    jednostka_spoza_pliku.zarzadzaj_automatycznie = True
    jednostka_spoza_pliku.save()

    autor_spoza_pliku.refresh_from_db()
    assert autor_spoza_pliku.aktualna_jednostka == jednostka_spoza_pliku

    # Odpinamy
    import_pracownikow_performed.odepnij_autorow_spoza_pliku()

    # Czy autor_spoza_pliku ma odpietą jednostkę?
    autor_spoza_pliku.refresh_from_db()
    assert autor_spoza_pliku.aktualna_jednostka is None


def test_odepnij_autorow_spoza_pliku_inny_autor_czy_nie_odepnie_nie_automatyczna(
    import_pracownikow_performed,
    autor_z_pliku,
    jednostka_z_pliku,
    autor_spoza_pliku,
    jednostka_spoza_pliku,
):
    autor_spoza_pliku.dodaj_jednostke(jednostka_spoza_pliku)
    jednostka_spoza_pliku.zarzadzaj_automatycznie = False
    jednostka_spoza_pliku.save()

    autor_spoza_pliku.refresh_from_db()
    assert autor_spoza_pliku.aktualna_jednostka == jednostka_spoza_pliku

    # Odpinamy
    import_pracownikow_performed.odepnij_autorow_spoza_pliku()

    # Czy autor_spoza_pliku ma odpietą jednostkę?
    autor_spoza_pliku.refresh_from_db()
    assert autor_spoza_pliku.aktualna_jednostka == jednostka_spoza_pliku



# Generated by Django 3.0.11 on 2021-03-15 19:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0247_funkcja_autora_pokazuj_za_nazwiskiem"),
    ]

    operations = [
        migrations.RunSQL(
            "CREATE INDEX IF NOT EXISTS bpp_autorzy_mat_8 ON bpp_autorzy_mat(dyscyplina_naukowa_id, rekord_id);",
            "DROP INDEX IF EXISTS bpp_autorzy_mat_8",
        )
    ]



# Generated by Django 3.2.15 on 2022-08-17 22:12

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0325_nullbooleanfield"),
    ]

    operations = [
        migrations.AddField(
            model_name="uczelnia",
            name="wymagaj_informacji_o_oplatach",
            field=models.BooleanField(
                default=True,
                help_text="Gdy zaznaczone, moduł 'Zgłaszanie publikacji' będzie wyświetlać użytkownikowi formularz "
                "informacji o opłatach za publikację w przypadku zgłaszania artykułu lub monografii. Gdy "
                "odznaczone, taki formularz nie bedzie wyświetlany, niezależnie od rodzaju zgłaszanej "
                "publikacji. ",
                verbose_name="Wymagaj inforamcji o opłatach",
            ),
        ),
        migrations.AlterField(
            model_name="patent",
            name="rok",
            field=models.IntegerField(
                db_index=True,
                help_text="Rok uwzględniany przy wyszukiwaniu i raportach\n        KBN/MNiSW)",
                validators=[django.core.validators.MinValueValidator(0)],
            ),
        ),
        migrations.AlterField(
            model_name="praca_doktorska",
            name="opl_pub_amount",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=20,
                null=True,
                verbose_name="Kwota brutto (zł)",
            ),
        ),
        migrations.AlterField(
            model_name="praca_doktorska",
            name="rok",
            field=models.IntegerField(
                db_index=True,
                help_text="Rok uwzględniany przy wyszukiwaniu i raportach\n        KBN/MNiSW)",
                validators=[django.core.validators.MinValueValidator(0)],
            ),
        ),
        migrations.AlterField(
            model_name="praca_habilitacyjna",
            name="opl_pub_amount",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=20,
                null=True,
                verbose_name="Kwota brutto (zł)",
            ),
        ),
        migrations.AlterField(
            model_name="praca_habilitacyjna",
            name="rok",
            field=models.IntegerField(
                db_index=True,
                help_text="Rok uwzględniany przy wyszukiwaniu i raportach\n        KBN/MNiSW)",
                validators=[django.core.validators.MinValueValidator(0)],
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle",
            name="opl_pub_amount",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=20,
                null=True,
                verbose_name="Kwota brutto (zł)",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle",
            name="rok",
            field=models.IntegerField(
                db_index=True,
                help_text="Rok uwzględniany przy wyszukiwaniu i raportach\n        KBN/MNiSW)",
                validators=[django.core.validators.MinValueValidator(0)],
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte",
            name="opl_pub_amount",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=20,
                null=True,
                verbose_name="Kwota brutto (zł)",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte",
            name="rok",
            field=models.IntegerField(
                db_index=True,
                help_text="Rok uwzględniany przy wyszukiwaniu i raportach\n        KBN/MNiSW)",
                validators=[django.core.validators.MinValueValidator(0)],
            ),
        ),
    ]



try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from model_bakery import baker

from django.contrib.auth.models import Group

from bpp.models import Autor, Jednostka, Uczelnia, Wydzial
from bpp.models.cache import Rekord
from bpp.models.system import Charakter_Formalny, Typ_Odpowiedzialnosci
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle_Autor
from bpp.tests.tests_legacy.testutil import SuperuserTestCase, UserTestCase
from bpp.tests.util import (
    any_autor,
    any_ciagle,
    any_doktorat,
    any_habilitacja,
    any_jednostka,
)
from bpp.util import rebuild_contenttypes
from bpp.views.browse import AutorView, AutorzyView


class TestRoot(UserTestCase):
    def setUp(self):
        Group.objects.get_or_create(name="wprowadzanie danych")

    def test_root(self):
        res = self.client.get("/")
        self.assertContains(res, "W systemie nie ma", status_code=200)

        Uczelnia.objects.create(nazwa="uczelnia", skrot="uu")
        res = self.client.get("/", follow=False)
        self.assertRedirects(
            res, reverse("bpp:browse_uczelnia", args=("uu",)), status_code=301
        )


class TestBrowse(UserTestCase):
    def setUp(self):
        Group.objects.get_or_create(name="wprowadzanie danych")

    def test_wydzial(self):
        u = Uczelnia.objects.create(nazwa="uczelnia", skrot="uu")
        Wydzial.objects.create(nazwa="wydzial", uczelnia=u)
        res = self.client.get(reverse("bpp:browse_uczelnia", args=("uu",)))
        self.assertContains(res, "Wybierz wydział", status_code=200)

    def test_jednostka(self):
        u = Uczelnia.objects.create(nazwa="uczelnia", skrot="uu")
        w = Wydzial.objects.create(nazwa="wydzial", uczelnia=u)
        j = Jednostka.objects.create(nazwa="jednostka", wydzial=w, uczelnia=u)

        res = self.client.get(reverse("bpp:browse_jednostka", args=(j.slug,)))
        self.assertContains(res, "jednostka", status_code=200)


class FakeUnauthenticatedUser:
    def is_authenticated(self):
        return False


class TestBrowseAutorzy(UserTestCase):
    def setUp(self):
        super().setUp()

        Group.objects.get_or_create(name="wprowadzanie danych")

        self.view = AutorzyView()

        class FakeRequest:
            GET = dict(search="Autor")

            def __init__(self):
                self.user = FakeUnauthenticatedUser()

        self.view.request = FakeRequest()
        self.autor = baker.make(Autor, nazwisko="Autor", imiona="Foo")
        self.view.kwargs = dict(literka="A")

    def test_get_queryset(self):
        q = self.view.get_queryset()
        self.assertEqual(list(q)[0], self.autor)

    def test_get_context_data(self):
        self.view.object_list = []
        d = self.view.get_context_data()
        self.assertEqual(d["wybrana"], "A")
        self.assertEqual(d["flt"], "Autor")


class TestBrowseAutor(UserTestCase):
    # fixtures = ['charakter_formalny.json', 'tytul.json',
    #             'typ_odpowiedzialnosci.json']

    def setUp(self):
        Group.objects.get_or_create(name="wprowadzanie danych")

        rebuild_contenttypes()

    def test_get_context_data(self):
        av = AutorView()
        av.object = baker.make(Autor)
        d = av.get_context_data()
        self.assertTrue("publikacje" in d["typy"])

    def test_habilitacyjna_doktorska(self):
        a = baker.make(Autor)
        kw = dict(autor=a, tytul_oryginalny="X", tytul="Y", uwagi="Z")
        any_doktorat(**kw)
        any_habilitacja(**kw)
        res = self.client.get(reverse("bpp:browse_autor", args=(a.slug,)))
        self.assertContains(res, "Praca doktorska")
        self.assertContains(res, "Praca habilitacyjna")

    def test_autor(self):
        a = baker.make(Autor)
        res = self.client.get(reverse("bpp:browse_autor", args=(a.slug,)))
        self.assertNotContains(res, "otwórz do edycji")


class TestBrowseAutorStaff(SuperuserTestCase):
    def setUp(self):
        super().setUp()
        Group.objects.get_or_create(name="wprowadzanie danych")

    def test_autor(self):
        a = baker.make(Autor)
        res = self.client.get(reverse("bpp:browse_autor", args=(a.slug,)))
        self.assertContains(res, "otwórz do edycji")


class TestOAI(UserTestCase):
    def setUp(self):
        super().setUp()

        rebuild_contenttypes()

        aut, ign = Typ_Odpowiedzialnosci.objects.get_or_create(
            skrot="aut.", nazwa="autor"
        )

        ch, ign = Charakter_Formalny.objects.get_or_create(
            skrot="AC", nazwa="Artykuł w czasopismie", nazwa_w_primo="Artykuł"
        )

        ch2, ign = Charakter_Formalny.objects.get_or_create(
            skrot="KOM", nazwa="Komentarz"
        )

        c = any_ciagle(tytul_oryginalny="Test foo bar", charakter_formalny=ch)

        c2 = any_ciagle(
            tytul_oryginalny="TEGO NIE BEDZIE bo nie ma nazwa_w_primo dla typu KOM",
            charakter_formalny=Charakter_Formalny.objects.get(skrot="KOM"),
        )

        a = any_autor()
        j = any_jednostka()

        for rekord in c, c2:
            Wydawnictwo_Ciagle_Autor.objects.create(
                rekord=rekord, autor=a, jednostka=j, typ_odpowiedzialnosci=aut
            )

        Rekord.objects.full_refresh()

        cnt = Rekord.objects.all().count()
        self.assertEqual(cnt, 2)

        self.c = c

    def test_get_record(self):
        url = reverse("bpp:oai")
        identifier = "oai:bpp.umlub.pl:Wydawnictwo_Ciagle/%s" % self.c.pk
        res = self.client.get(
            url,
            data={
                "verb": "GetRecord",
                "metadataPrefix": "oai_dc",
                "identifier": identifier,
            },
        )
        self.assertContains(res, "foo", status_code=200)

    def test_list_records(self):
        url = reverse("bpp:oai")

        res = self.client.get(
            url, data={"verb": "ListRecords", "metadataPrefix": "oai_dc"}
        )
        self.assertContains(res, "foo", status_code=200)
        self.assertNotContains(res, "TEGO NIE BEDZIE", status_code=200)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-16 11:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0126_zewnetrzna_baza_danych'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='wydawnictwo_ciagle_zewnetrzna_baza_danych',
            options={'verbose_name': 'powiązanie wydawnictwa ciągłego z zewnętrznymi bazami danych', 'verbose_name_plural': 'powiązania wydawnictw ciągłych z zewnętrznymi bazami danych'},
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle_zewnetrzna_baza_danych',
            name='info',
            field=models.CharField(blank=True, max_length=512, null=True, verbose_name='Informacje dodatkowe'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations
import bpp.fields


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0032_auto_20151001_1413'),
    ]

    operations = [
        migrations.AddField(
            model_name='praca_doktorska',
            name='doi',
            field=bpp.fields.DOIField(max_length=2048, blank=True, help_text=b'Digital Object Identifier (DOI)', null=True, verbose_name=b'DOI', db_index=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='praca_habilitacyjna',
            name='doi',
            field=bpp.fields.DOIField(max_length=2048, blank=True, help_text=b'Digital Object Identifier (DOI)', null=True, verbose_name=b'DOI', db_index=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='doi',
            field=bpp.fields.DOIField(max_length=2048, blank=True, help_text=b'Digital Object Identifier (DOI)', null=True, verbose_name=b'DOI', db_index=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='doi',
            field=bpp.fields.DOIField(max_length=2048, blank=True, help_text=b'Digital Object Identifier (DOI)', null=True, verbose_name=b'DOI', db_index=True),
            preserve_default=True,
        ),
    ]



import pytest
from model_bakery import baker

from pbn_api.integrator import (
    wydawnictwa_ciagle_do_synchronizacji,
    wydawnictwa_zwarte_do_synchronizacji,
)

from bpp.const import PBN_MIN_ROK
from bpp.models import (
    Charakter_Formalny,
    Jezyk,
    Status_Korekty,
    Typ_KBN,
    Wydawnictwo_Ciagle,
    Wydawnictwo_Zwarte,
)


@pytest.mark.django_db
def test_wydawnictwa_zwarte_do_synchronizacji(pbn_charakter_formalny, pbn_jezyk):
    wejda = []
    nie_wejda = []

    required = {
        "status_korekty": baker.make(Status_Korekty),
        "typ_kbn": baker.make(Typ_KBN),
    }

    wejda.append(
        Wydawnictwo_Zwarte.objects.create(
            tytul_oryginalny="A",
            e_isbn="jest",
            doi="jest",
            rok=PBN_MIN_ROK,
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )

    wejda.append(
        Wydawnictwo_Zwarte.objects.create(
            tytul_oryginalny="B",
            isbn="jest",
            doi="jest",
            rok=PBN_MIN_ROK,
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )

    nadrzedne_www = Wydawnictwo_Zwarte.objects.create(
        tytul_oryginalny="C",
        rok=PBN_MIN_ROK,
        www="jest",
        charakter_formalny=pbn_charakter_formalny,
        jezyk=pbn_jezyk,
        **required
    )
    nadrzedne_public_www = Wydawnictwo_Zwarte.objects.create(
        rok=PBN_MIN_ROK,
        www="jest",
        charakter_formalny=pbn_charakter_formalny,
        jezyk=pbn_jezyk,
        **required
    )

    wejda.append(
        Wydawnictwo_Zwarte.objects.create(
            tytul_oryginalny="D",
            isbn="jest",
            rok=PBN_MIN_ROK,
            wydawnictwo_nadrzedne=nadrzedne_www,
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )
    wejda.append(
        Wydawnictwo_Zwarte.objects.create(
            tytul_oryginalny="E",
            isbn="jest",
            rok=PBN_MIN_ROK,
            wydawnictwo_nadrzedne=nadrzedne_public_www,
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )

    nie_wejda.append(
        # Charakter formalny bez odpowiednika
        Wydawnictwo_Zwarte.objects.create(
            tytul_oryginalny="F",
            doi="jest",
            isbn="jest",
            rok=PBN_MIN_ROK,
            charakter_formalny=baker.make(Charakter_Formalny, rodzaj_pbn=None),
            jezyk=pbn_jezyk,
            **required
        )
    )

    nie_wejda.append(
        # Brak ISBN oraz E-ISBN
        Wydawnictwo_Zwarte.objects.create(
            tytul_oryginalny="G",
            doi="jest",
            rok=PBN_MIN_ROK,
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )

    nie_wejda.append(
        # Brak www oraz public_www oraz DOI
        Wydawnictwo_Zwarte.objects.create(
            tytul_oryginalny="H",
            rok=PBN_MIN_ROK,
            charakter_formalny=pbn_charakter_formalny,
            isbn="jest",
            jezyk=pbn_jezyk,
            **required
        )
    )

    nie_wejda.append(
        # Jezyk bez odpowiednika w PBN
        Wydawnictwo_Zwarte.objects.create(
            tytul_oryginalny="I",
            rok=PBN_MIN_ROK,
            charakter_formalny=pbn_charakter_formalny,
            isbn="jest",
            jezyk=baker.make(Jezyk, pbn_uid=None),
            **required
        )
    )

    nie_wejda.append(
        Wydawnictwo_Zwarte.objects.create(
            # Rok za wczesny
            tytul_oryginalny="J",
            e_isbn="jest",
            doi="jest",
            rok=PBN_MIN_ROK - 10,
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )

    res = list(wydawnictwa_zwarte_do_synchronizacji())
    for elem in wejda:
        assert elem in res, elem.tytul_oryginalny
    for elem in nie_wejda:
        assert elem not in res, elem.tytul_oryginalny


@pytest.mark.django_db
def test_wydawnictwa_ciagle_do_synchronizacji(pbn_charakter_formalny, pbn_jezyk):
    wejda = []
    nie_wejda = []

    required = {
        "status_korekty": baker.make(Status_Korekty),
        "typ_kbn": baker.make(Typ_KBN),
    }

    wejda.append(
        Wydawnictwo_Ciagle.objects.create(
            tytul_oryginalny="A",
            doi="jest",
            rok=PBN_MIN_ROK,
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )

    wejda.append(
        Wydawnictwo_Ciagle.objects.create(
            tytul_oryginalny="B",
            www="jest",
            rok=PBN_MIN_ROK,
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )
    wejda.append(
        Wydawnictwo_Ciagle.objects.create(
            tytul_oryginalny="B",
            public_www="jest",
            rok=PBN_MIN_ROK,
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )

    nie_wejda.append(
        # Charakter formalny bez odpowiednika
        Wydawnictwo_Ciagle.objects.create(
            tytul_oryginalny="F",
            doi="jest",
            rok=PBN_MIN_ROK,
            charakter_formalny=baker.make(Charakter_Formalny, rodzaj_pbn=None),
            jezyk=pbn_jezyk,
            **required
        )
    )

    nie_wejda.append(
        # Brak www oraz public_www oraz DOI
        Wydawnictwo_Ciagle.objects.create(
            tytul_oryginalny="H",
            rok=PBN_MIN_ROK,
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )

    nie_wejda.append(
        # Jezyk bez odpowiednika w PBN
        Wydawnictwo_Ciagle.objects.create(
            tytul_oryginalny="I",
            rok=PBN_MIN_ROK,
            charakter_formalny=pbn_charakter_formalny,
            doi="jest",
            jezyk=baker.make(Jezyk, pbn_uid=None),
            **required
        )
    )

    nie_wejda.append(
        Wydawnictwo_Ciagle.objects.create(
            # Rok za wczesny
            tytul_oryginalny="J",
            rok=PBN_MIN_ROK - 10,
            doi="jest",
            charakter_formalny=pbn_charakter_formalny,
            jezyk=pbn_jezyk,
            **required
        )
    )

    res = list(wydawnictwa_ciagle_do_synchronizacji())
    for elem in wejda:
        assert elem in res, elem.tytul_oryginalny
    for elem in nie_wejda:
        assert elem not in res, elem.tytul_oryginalny



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-19 23:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('bpp', '0096_adnotacje_not_null'),
    ]

    operations = [
        migrations.CreateModel(
            name='Nagroda',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('rok_przyznania', models.PositiveIntegerField()),
                ('uzasadnienie', models.CharField(blank=True, default='', max_length=512)),
                ('adnotacja', models.CharField(blank=True, default='', max_length=100)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'nagroda',
                'verbose_name_plural': 'nagrody',
            },
        ),
        migrations.CreateModel(
            name='OrganPrzyznajacyNagrody',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nazwa', models.CharField(max_length=512, unique=True)),
                ('skrot', models.CharField(max_length=128, unique=True)),
            ],
            options={
                'verbose_name': 'Organ przyznający nagrodę',
                'verbose_name_plural': 'Organy przyznające nagrody',
            },
        ),
        migrations.AddField(
            model_name='nagroda',
            name='organ_przyznajacy',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.OrganPrzyznajacyNagrody'),
        ),
        migrations.AlterUniqueTogether(
            name='nagroda',
            unique_together=set([('content_type', 'object_id', 'organ_przyznajacy', 'rok_przyznania')]),
        ),
    ]



from multiseek.logic import Ordering, create_registry

from .fields import *  # noqa
from .reports import multiseek_report_types

from bpp.models import Rekord

registry = create_registry(
    Rekord,
    *multiseek_fields,  # noqa
    ordering=[
        Ordering("", "(nieistotne)"),
        Ordering("tytul_oryginalny", "tytuł oryginalny"),
        Ordering("rok", "rok"),
        Ordering("impact_factor", "impact factor"),
        Ordering("liczba_cytowan", "liczba cytowań"),
        Ordering("liczba_autorow", "liczba autorów"),
        Ordering("punkty_kbn", "punkty PK"),
        Ordering("charakter_formalny__nazwa", "charakter formalny"),
        Ordering("typ_kbn__nazwa", "typ KBN"),
        Ordering("zrodlo_lub_nadrzedne", "źródło/wyd.nadrz."),
        Ordering("utworzono", "utworzono"),
        Ordering("ostatnio_zmieniony", "ostatnio zmieniony"),
    ],
    default_ordering=["-rok", "", ""],
    report_types=multiseek_report_types
)


def _get_fields(self, request):
    """Ta funkcja sortuje pola zgodnie z ustawieniem pola ui_order (czyli bazodanowy
    sort_order obiektu BppMultiseekVisibility) i zwraca je w kolejności.
    """
    return sorted(
        [x for x in self.fields if x.enabled(request)], key=lambda x: x.ui_order
    )


registry.get_fields = lambda request, registry=registry: _get_fields(registry, request)



from zglos_publikacje.models import Obslugujacy_Zgloszenia_Wydzialow

from django.contrib import admin


@admin.register(Obslugujacy_Zgloszenia_Wydzialow)
class Obslugujacy_Zgloszenia_WydzialowAdmin(admin.ModelAdmin):
    list_select_related = ["user", "wydzial"]
    list_display = ["user", "wydzial"]



import pytest
from django.urls import reverse
from model_bakery import baker

from pbn_api.models import Publication

from bpp.models import Wydawnictwo_Zwarte
from bpp.tests import normalize_html

TEST_PBN_ID = 50000


@pytest.mark.parametrize(
    "fld,value",
    [
        ("pbn_uid", TEST_PBN_ID),
        ("doi", "10.10/123123"),
        ("www", "https://foobar.pl"),
        ("public_www", "https://foobar.pl"),
    ],
)
def test_Wydawnictwo_Zwarte_Admin_sprawdz_duplikaty_www_doi(admin_app, fld, value):
    if fld == "pbn_uid":
        value = baker.make(Publication, pk=TEST_PBN_ID)

    baker.make(Wydawnictwo_Zwarte, rok=2020, **{fld: value})
    w2 = baker.make(Wydawnictwo_Zwarte, rok=2020)
    if fld == "pbn_uid":
        value = TEST_PBN_ID  # baker.make(Publication, pk=TEST_PBN_ID)

    url = "admin:bpp_wydawnictwo_zwarte_change"
    page = admin_app.get(reverse(url, args=(w2.pk,)))

    if fld == "pbn_uid":
        page.forms["wydawnictwo_zwarte_form"][fld].force_value(value)
    else:
        page.forms["wydawnictwo_zwarte_form"][fld].value = value
    res = page.forms["wydawnictwo_zwarte_form"].submit().maybe_follow()

    assert "inne rekordy z identycznym polem" in normalize_html(
        res.content.decode("utf-8")
    )



# -*- encoding: utf-8 -*-
import time

from selenium.webdriver.support.expected_conditions import staleness_of
from selenium.webdriver.support.ui import WebDriverWait

VERY_SHORT_WAIT_TIME = 1
SHORT_WAIT_TIME = 5
LONG_WAIT_TIME = 10
DEFAULT_WAIT_TIME = SHORT_WAIT_TIME
PAGE_LOAD_WAIT_TIME = LONG_WAIT_TIME


def wait_for(condition_function, max_seconds=DEFAULT_WAIT_TIME):
    start_time = time.time()
    while time.time() < start_time + max_seconds:
        if condition_function():
            return True
        else:
            time.sleep(0.1)
    raise TimeoutError("Timeout waiting for {}".format(condition_function.__name__))


class wait_for_page_load(object):
    def __init__(self, browser, max_seconds=PAGE_LOAD_WAIT_TIME):
        self.browser = browser
        self.max_seconds = max_seconds

    def __enter__(self):
        self.old_page = self.browser.find_by_tag("html").first._element

    def __exit__(self, *_):
        WebDriverWait(self.browser, self.max_seconds).until(
            lambda x: staleness_of(self.old_page)
        )

        WebDriverWait(self.browser, self.max_seconds).until(
            lambda browser: browser.execute_script("return document.readyState")
            == "complete"
        )

        WebDriverWait(self.browser, self.max_seconds).until(
            lambda browser: not browser.find_by_tag("body").is_empty()
        )


def wait_for_websocket_connection(browser):
    WebDriverWait(browser, DEFAULT_WAIT_TIME).until(
        lambda browser: browser.execute_script(
            "return bppNotifications.chatSocket.readyState"
        )
        == 1
    )



import django_filters
from rest_framework import viewsets

from api_v1.serializers.zrodlo import Rodzaj_ZrodlaSerializer, ZrodloSerializer

from bpp.models import Rodzaj_Zrodla, Zrodlo


class ZrodloFilterSet(django_filters.rest_framework.FilterSet):
    ostatnio_zmieniony = django_filters.DateTimeFromToRangeFilter("ostatnio_zmieniony")

    class Meta:
        fields = ["ostatnio_zmieniony"]
        model = Zrodlo


class ZrodloViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Zrodlo.objects.all()
    serializer_class = ZrodloSerializer
    filterset_class = ZrodloFilterSet


class Rodzaj_ZrodlaViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Rodzaj_Zrodla.objects.all()
    serializer_class = Rodzaj_ZrodlaSerializer



from raport_slotow.core import (
    autorzy_z_dyscyplinami,
    autorzy_z_punktami,
    autorzy_zerowi,
)


def test_autorzy_z_dyscyplinami_parametr(autor_z_dyscyplina, rok):
    assert autorzy_z_dyscyplinami(od_roku=rok, do_roku=rok).count() == 1


def test_autorzy_z_dyscyplinami_bez_parametru(autor_z_dyscyplina, rok):
    assert autorzy_z_dyscyplinami().count() == 1


def test_autorzy_z_dyscyplinami_poza_zakr(autor_z_dyscyplina, rok):
    assert autorzy_z_dyscyplinami(od_roku=rok + 10).count() == 0
    assert autorzy_z_dyscyplinami(do_roku=rok + 10).count() == 1


def test_autorzy_z_punktami_autor_nie_ma(autor_jan_nowak):
    assert autorzy_z_punktami().count() == 0


def test_autorzy_z_punktami_autor_nadal_nie_ma(autor_z_dyscyplina):
    assert autorzy_z_punktami().count() == 0


def test_autorzy_z_punktami_autor_ma(praca_z_dyscyplina):
    assert autorzy_z_punktami().count() == 1


def test_autorzy_zerowi(rekord_slotu, autor_z_dyscyplina):
    assert autorzy_zerowi().count() == 1


def test_autorzy_zerowi_rok_powyzej(rekord_slotu, autor_jan_nowak, rok):
    assert autorzy_zerowi(od_roku=rok + 20).count() == 0



from http.server import HTTPServer
from uuid import uuid4

import pytest
from model_bakery import baker

from pbn_api.client import PBN_GET_LANGUAGES_URL, PBNClient
from pbn_api.models import Institution, Language, Publication, Scientist
from pbn_api.tests.utils import MockTransport

from bpp import const
from bpp.const import RODZAJ_PBN_KSIAZKA
from bpp.models import Charakter_Formalny, Jezyk, Uczelnia, Wydawnictwo_Ciagle

MOCK_RETURNED_MONGODB_DATA = dict(
    status="foo",
    verificationLevel="bar",
    verified=True,
    versions=[{"current": True, "baz": "quux"}],
    mongoId="123",
)


@pytest.fixture
def pbn_client():
    transport = MockTransport()
    return PBNClient(transport=transport)


@pytest.fixture
def pbn_language():
    return Language.objects.create(code="pl", language={"lol": "XD"})


def _zrob_autora_pbn(autor):
    autor.pbn_uid = baker.make(Scientist)
    autor.save()
    return autor


@pytest.fixture
def pbn_autor(autor_jan_nowak):
    return _zrob_autora_pbn(autor_jan_nowak)


@pytest.fixture
def pbn_jednostka(jednostka):
    jednostka.pbn_uid = baker.make(Institution)
    jednostka.save()
    return jednostka


@pytest.fixture
def pbn_jezyk(jezyki, pbn_language):
    polski = Jezyk.objects.get(nazwa="polski")
    polski.pbn_uid = pbn_language
    polski.save()
    return polski


def _zrob_wydawnictwo_pbn(
    wydawnictwo_ciagle, jezyk, rodzaj_pbn=const.RODZAJ_PBN_ARTYKUL
):
    cf = wydawnictwo_ciagle.charakter_formalny
    cf.rodzaj_pbn = rodzaj_pbn
    cf.save()

    wydawnictwo_ciagle.jezyk = jezyk

    wydawnictwo_ciagle.doi = "123;.123/doi"
    wydawnictwo_ciagle.save()


@pytest.fixture
def pbn_wydawnictwo_ciagle(wydawnictwo_ciagle, pbn_jezyk) -> Wydawnictwo_Ciagle:
    _zrob_wydawnictwo_pbn(wydawnictwo_ciagle, pbn_jezyk)
    return wydawnictwo_ciagle


@pytest.fixture
def pbn_wydawnictwo_ciagle_z_autorem_z_dyscyplina(
    pbn_wydawnictwo_ciagle, autor_z_dyscyplina, pbn_jednostka
) -> Wydawnictwo_Ciagle:
    _zrob_autora_pbn(autor_z_dyscyplina.autor)
    pbn_wydawnictwo_ciagle.dodaj_autora(
        autor_z_dyscyplina.autor,
        pbn_jednostka,
        dyscyplina_naukowa=autor_z_dyscyplina.dyscyplina_naukowa,
    )
    return pbn_wydawnictwo_ciagle


@pytest.fixture
def pbn_wydawnictwo_zwarte_z_autorem_z_dyscyplina(
    pbn_wydawnictwo_zwarte, autor_z_dyscyplina, pbn_jednostka
) -> Wydawnictwo_Ciagle:
    _zrob_autora_pbn(autor_z_dyscyplina.autor)
    pbn_wydawnictwo_zwarte.dodaj_autora(
        autor_z_dyscyplina.autor,
        pbn_jednostka,
        dyscyplina_naukowa=autor_z_dyscyplina.dyscyplina_naukowa,
    )
    return pbn_wydawnictwo_zwarte


@pytest.fixture
def pbn_rozdzial_z_autorem_z_dyscyplina(
    pbn_wydawnictwo_zwarte, autor_z_dyscyplina, pbn_jednostka
) -> Wydawnictwo_Ciagle:
    _zrob_autora_pbn(autor_z_dyscyplina.autor)
    pbn_wydawnictwo_zwarte.dodaj_autora(
        autor_z_dyscyplina.autor,
        pbn_jednostka,
        dyscyplina_naukowa=autor_z_dyscyplina.dyscyplina_naukowa,
    )
    cf = pbn_wydawnictwo_zwarte.charakter_formalny
    cf.rodzaj_pbn = const.RODZAJ_PBN_ROZDZIAL
    cf.save()
    return pbn_wydawnictwo_zwarte


@pytest.fixture
def pbn_charakter_formalny() -> Charakter_Formalny:
    return baker.make(Charakter_Formalny, rodzaj_pbn=RODZAJ_PBN_KSIAZKA)


@pytest.fixture
def pbn_wydawnictwo_zwarte(wydawnictwo_zwarte, pbn_jezyk):
    cf = wydawnictwo_zwarte.charakter_formalny
    cf.rodzaj_pbn = const.RODZAJ_PBN_ARTYKUL
    cf.save()

    wydawnictwo_zwarte.jezyk = pbn_jezyk

    wydawnictwo_zwarte.doi = "123;.123/doi"
    wydawnictwo_zwarte.save()

    return wydawnictwo_zwarte


@pytest.fixture
def pbn_wydawnictwo_zwarte_ksiazka(pbn_wydawnictwo_zwarte):
    cf = pbn_wydawnictwo_zwarte.charakter_formalny
    cf.rodzaj_pbn = const.RODZAJ_PBN_KSIAZKA
    cf.save()

    return pbn_wydawnictwo_zwarte


@pytest.fixture
def pbn_wydawnictwo_zwarte_rozdzial(pbn_wydawnictwo_zwarte):
    cf = pbn_wydawnictwo_zwarte.charakter_formalny
    cf.rodzaj_pbn = const.RODZAJ_PBN_ROZDZIAL
    cf.save()

    return pbn_wydawnictwo_zwarte


@pytest.fixture
def pbn_publication(db):
    return Publication.objects.create(mongoId=123, versions={})


@pytest.fixture
def pbn_wydawnictwo_zwarte_z_charakterem(
    pbn_wydawnictwo_zwarte, pbn_charakter_formalny
):
    pbn_wydawnictwo_zwarte.charakter_formalny = pbn_charakter_formalny
    pbn_wydawnictwo_zwarte.save()
    return pbn_wydawnictwo_zwarte


@pytest.fixture
def pbn_uczelnia(pbn_client) -> Uczelnia:
    uczelnia = baker.make(
        Uczelnia,
    )

    uczelnia.pbn_client = lambda *args, **kw: pbn_client
    pbn_client.transport.return_values[PBN_GET_LANGUAGES_URL] = {"1": "23"}

    uczelnia.pbn_integracja = True
    uczelnia.pbn_aktualizuj_na_biezaco = True
    uczelnia.pbn_app_name = "FOO"
    uczelnia.pbn_app_token = "BAR"
    uczelnia.pbn_api_root = "https://localhost/pbn/mockapi/v1/"
    uczelnia.save()

    return uczelnia


@pytest.fixture
def pbn_serwer(pbn_uczelnia, httpserver: HTTPServer):
    pbn_uczelnia.pbn_api_root = httpserver.url_for("/")
    pbn_uczelnia.save()
    return httpserver


def pbn_general_json(
    object_,
    mongoId=None,
    status="ACTIVE",
    verified=True,
):
    return {
        "mongoId": mongoId or str(uuid4()),
        "status": status,
        "verificationLevel": "MODERATOR",
        "verified": verified,
        "versions": [{"current": True, "object": object_}],
    }


def pbn_journal_json(
    title="tytul journalu",
    websiteLink="http://onet.pl",
    issn="1234-1234",
    eissn="4567-4567",
    mongoId=None,
    status="ACTIVE",
    verified=True,
):
    obj = {"title": title, "websiteLink": websiteLink}
    if issn:
        obj.update({"issn": issn})
    if eissn:
        obj.update({"eissn": eissn})

    return pbn_general_json(obj, mongoId=mongoId, status=status, verified=verified)


def pbn_publication_json(
    year,
    title="tytul pracy",
    isbn=None,
    doi=None,
    mongoId=None,
    status="ACTIVE",
    verified=True,
):
    obj = {"title": title, "year": year}
    if isbn:
        obj.update({"isbn": isbn})
    if doi:
        obj.update({"doi": doi})

    return pbn_general_json(obj, mongoId=mongoId, status=status, verified=verified)


def pbn_pageable_json(content):
    return {
        "content": content,
        "first": True,
        "last": True,
        "number": 0,
        "numberofElements": len(content),
        "pageable": {
            "offset": 0,
            "pageNumber": 0,
            "pageSize": len(content),
            "paged": True,
            "sort": {"sorted": False, "unsorted": True},
            "unpaged": False,
        },
        "size": len(content),
        "sort": {"sorted": False, "unsorted": True},
        "totalElements": len(content),
        "totalPages": 1,
    }



from braces.views import GroupRequiredMixin

from import_list_if.forms import NowyImportForm
from import_list_if.models import ImportListIf
from long_running.views import (
    CreateLongRunningOperationView,
    LongRunningDetailsView,
    LongRunningOperationsView,
    LongRunningResultsView,
    LongRunningRouterView,
    RestartLongRunningOperationView,
)


class BaseImportListIfMixin(GroupRequiredMixin):
    group_required = "wprowadzanie danych"
    model = ImportListIf


class ListaImportowView(BaseImportListIfMixin, LongRunningOperationsView):
    pass


class NowyImportView(BaseImportListIfMixin, CreateLongRunningOperationView):
    form_class = NowyImportForm


class ImportListIfRouterView(BaseImportListIfMixin, LongRunningRouterView):
    redirect_prefix = "import_list_if:import_list_if"


class ImportListIfDetailsView(BaseImportListIfMixin, LongRunningDetailsView):
    pass


class ImportListIfResultsView(BaseImportListIfMixin, LongRunningResultsView):
    pass


class RestartImportView(BaseImportListIfMixin, RestartLongRunningOperationView):
    pass



# Generated by Django 3.0.11 on 2021-04-27 22:11

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0263_auto_20210427_2137"),
    ]

    operations = [
        migrations.AddField(
            model_name="uczelnia",
            name="pbn_api_user",
            field=models.ForeignKey(
                blank=True,
                help_text="Użytkownik po stronie BPP który bedzie odpowiedzialny za operacje przeprowadzane w tle przez procesy działające na serwerze i pobierające dane z PBN np w nocy. Jeżeli ten użytkownik dokona autoryzacji w PBN za pomocą przeglądarki, to możliwe będzie również aktualizowanie (wgrywanie) rekordów przez niego na serwer PBN. ",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to=settings.AUTH_USER_MODEL,
                verbose_name="Użytkownik BPP dla PBN API",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pbn_aktualizuj_na_biezaco",
            field=models.BooleanField(
                default=False,
                help_text="Aktualizuj rekordy w PBN przy każdym zapisie rekordu. Może spowolnić prace. W przypadku braku dostępu do serwerów PBN nie uniemożliwia edycji rekordów. ",
                verbose_name="Aktualizuj PBN na bieżąco",
            ),
        ),
    ]



from model_bakery import baker

from ewaluacja2021.core.util import get_lista_prac
from ewaluacja2021.models import IloscUdzialowDlaAutora, LiczbaNDlaUczelni

from bpp.models import Autor_Dyscyplina, Wydawnictwo_Ciagle


def test_get_lista_prac_zakres_lat(
    autor_jan_nowak, dyscyplina1, jednostka, uczelnia, denorms, typy_odpowiedzialnosci
):
    """Sprawdza, czy lista prac odrzuca prace spoza zakresu 2017-2021"""

    # Zrob dane testowe od 2015 do 2025

    LiczbaNDlaUczelni.objects.create(
        dyscyplina_naukowa=dyscyplina1, uczelnia=uczelnia, liczba_n=100
    )
    IloscUdzialowDlaAutora.objects.create(
        autor=autor_jan_nowak,
        ilosc_udzialow=10,
        ilosc_udzialow_monografie=10,
        dyscyplina_naukowa=dyscyplina1,
    )

    for ROK in range(2015, 2026):
        Autor_Dyscyplina.objects.create(
            autor=autor_jan_nowak, rok=ROK, dyscyplina_naukowa=dyscyplina1
        )

        wc: Wydawnictwo_Ciagle = baker.make(
            Wydawnictwo_Ciagle,
            rok=ROK,
            punkty_kbn=5,
            tytul_oryginalny=f"Test 123 - praca za rok {ROK}",
        )
        wc.dodaj_autora(autor_jan_nowak, jednostka, dyscyplina_naukowa=dyscyplina1)

    denorms.flush()

    assert (len(list(get_lista_prac(dyscyplina1.nazwa)))) == 5



# -*- encoding: utf-8 -*-

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from bpp.tests.tests_legacy.testutil import WebTestCase, UserTestCase


class TestMultiseekAnonymous(WebTestCase):
    def test_multiseek(self):
        res = self.client.get(reverse("multiseek:index"))
        self.assertNotContains(res, "Adnotacje", status_code=200)


class TestMultiseekLoggedIn(UserTestCase):
    def test_multiseek(self):
        res = self.client.get(reverse("multiseek:index"))
        self.assertContains(res, "Adnotacje", status_code=200)



# -*- encoding: utf-8 -*-

"""
Ten moduł zawiera rozwiązania dla języka polskiego.
"""

czasownik_byc = dict(
    czas_przeszly=dict(
        K='była',
        M='był',
        default='był(a)'
    ),
    czas_terazniejszy=dict(
        K='jest',
        M='jest',
        default='jest'
    )
)

def warianty_zapisanego_nazwiska(p_imiona, p_nazwisko, poprzednie_nazwiska):
    imiona = p_imiona.replace("*", "")
    nazwisko = p_nazwisko.replace("*", "")
    # Pozbądź się pustych imion
    imiona = [x for x in imiona.split(" ") if x]
    # Sprawdź, czy mamy podwójne nazwiska
    nazwiska = [x for x in nazwisko.replace("-", " ").split(" ") if x]
    if len(nazwiska)>1:
        nazwiska = [nazwisko,] + nazwiska

    def wersje_imienia(imie):
        buf = [imie, ]
        if len(imie)>2:
            buf.append(imie[0] + "[" + imie[1:] + "]")
            buf.append(imie[0] + ".")
        else:
            buf.append(imie)
            buf.append(imie)
        return buf

    for i in zip(*tuple([wersje_imienia(imie) for imie in imiona])):
        for n in nazwiska:
            yield " ".join(list(i)) + " " + n

    # Wersja, w której z autora z wieloma imionami robimy autora z jednym imieniem,
    # czyli np. Stanisław J. Czuczwar => Stanisław Czuczwar
    if len(imiona)>1:
        for elem in warianty_zapisanego_nazwiska(imiona[0], p_nazwisko, poprzednie_nazwiska):
            yield elem

        for elem in warianty_zapisanego_nazwiska(imiona[1], p_nazwisko, poprzednie_nazwiska):
            yield elem

    if poprzednie_nazwiska:
        for elem in warianty_zapisanego_nazwiska(p_imiona, poprzednie_nazwiska, None):
            yield elem


# -*- coding: utf-8 -*-


from django.db import models, migrations
import datetime


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0002_auto_20141020_1738'),
    ]

    operations = [
        migrations.AlterField(
            model_name='patent',
            name='utworzono',
            field=models.DateTimeField(default=datetime.datetime(1970, 1, 1, 0, 0), auto_now_add=True, verbose_name=b'Utworzono', db_index=True),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='utworzono',
            field=models.DateTimeField(default=datetime.datetime(1970, 1, 1, 0, 0), auto_now_add=True, verbose_name=b'Utworzono', db_index=True),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='utworzono',
            field=models.DateTimeField(default=datetime.datetime(1970, 1, 1, 0, 0), auto_now_add=True, verbose_name=b'Utworzono', db_index=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='utworzono',
            field=models.DateTimeField(default=datetime.datetime(1970, 1, 1, 0, 0), auto_now_add=True, verbose_name=b'Utworzono', db_index=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='utworzono',
            field=models.DateTimeField(default=datetime.datetime(1970, 1, 1, 0, 0), auto_now_add=True, verbose_name=b'Utworzono', db_index=True),
        ),
    ]



# -*- encoding: utf-8 -*-

# -*- encoding: utf-8 -*-
import datetime

from django.contrib.contenttypes.models import ContentType

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from django.db.models.aggregates import Min
from django.http.response import HttpResponse, HttpResponseServerError
from django.utils import timezone
from django.utils.timezone import make_naive
from django.views.generic.base import View
from moai.oai import OAIServerFactory
from moai.server import FeedConfig

from bpp.models import Rekord, Uczelnia


class CacheMetadata:
    def __init__(self, orig):
        self.orig = orig

    def get(self, item, default):
        def ifhas(attr):
            if hasattr(item, attr):
                return [
                    getattr(item, attr),
                ]
            return default

        if item == "title":
            if self.orig.tytul:
                return [self.orig.tytul_oryginalny, self.orig.tytul]
            return [
                self.orig.tytul_oryginalny,
            ]

        if item == "language":
            if hasattr(self.orig, "jezyk"):
                return [
                    self.orig.jezyk.nazwa,
                ]

        if item == "creator":
            return self.orig.opis_bibliograficzny_autorzy_cache or []

        if item == "date":
            return [
                str(self.orig.rok),
            ]

        if item == "publisher":
            return ifhas("wydawnictwo")

        if item == "subject":
            if hasattr(item, "slowa_kluczowe"):
                return u", ".join(o.name for o in item.slowa_kluczowe.all())

        if item == "source":
            src = []
            if getattr(self.orig, "zrodlo_id", None) is not None:
                src.append(
                    "%s %s %s"
                    % (
                        self.orig.zrodlo.nazwa,
                        self.orig.informacje,
                        self.orig.szczegoly,
                    )
                )
            else:
                if self.orig.informacje or self.orig.szczegoly:
                    src.append("%s %s" % (self.orig.informacje, self.orig.szczegoly))

            if self.orig.www:
                src.append(self.orig.www)

            return src

        if item == "type":
            return [
                self.orig.charakter_formalny.nazwa_w_primo,
            ]

        return default


def get_dc_ident(model, obj_pk):
    return "oai:bpp.umlub.pl:%s/%s" % (model, str(obj_pk))


class BPPOAIDatabase(object):
    def __init__(self, original):
        self.original = original

    def get_set(self, oai_id):
        if oai_id == 1:
            return {
                "id": "1",
                "name": "wszystkie",
                "description": "wszystkie rekordy",
                "hidden": False,
            }

        return

        row = self._sets.select(self._sets.c.set_id == oai_id).execute().fetchone()

        if row is None:
            return

    # mpasternak, 24.02.2021 -- czy ten kod w ogóle był uzywany?
    # def get_setrefs(self, oai_id, include_hidden_sets=False):
    #     return [1]
    #
    #     set_ids = []
    #     query = sql.select([self._setrefs.c.set_id])
    #     query.append_whereclause(self._setrefs.c.record_id == oai_id)
    #     if not include_hidden_sets:
    #         query.append_whereclause(
    #             sql.and_(
    #                 self._sets.c.set_id == self._setrefs.c.set_id,
    #                 self._sets.c.hidden == include_hidden_sets,
    #             )
    #         )
    #
    #     for row in query.execute():
    #         set_ids.append(row[0])
    #     set_ids.sort()
    #     return set_ids

    def record_count(self):
        return self.original.count()
        return self.original.objects.all().count()

    def set_count(self):
        return 1

    def oai_sets(self, offset=0, batch_size=20):
        yield {"id": "1", "name": "wszystkie", "description": "wszystkie rekordy"}

    def oai_earliest_datestamp(self):
        r = self.original.aggregate(Min("ostatnio_zmieniony"))
        value = r["ostatnio_zmieniony__min"]
        if value is None:
            return datetime.datetime.fromtimestamp(0)
        return value.replace(tzinfo=None)

    def oai_query(
        self,
        offset=0,
        batch_size=20,
        needed_sets=None,
        disallowed_sets=None,
        allowed_sets=None,
        from_date=None,
        until_date=None,
        identifier=None,
    ):

        needed_sets = needed_sets or []
        disallowed_sets = disallowed_sets or []
        allowed_sets = allowed_sets or []
        if batch_size < 0:
            batch_size = 0

        # make sure until date is set, and not in future
        if until_date is None or until_date > datetime.datetime.utcnow():
            until_date = timezone.now()

        query = self.original.order_by("-ostatnio_zmieniony")

        # filter dates
        query = query.filter(ostatnio_zmieniony__lte=until_date)

        if identifier is not None:
            ident = identifier.split(":")
            assert ident[0] == "oai"
            assert ident[1] == "bpp.umlub.pl"

            ident = ident[2].split("/")
            klass = ident[0].lower()

            content_type_id = ContentType.objects.get(app_label="bpp", model=klass).pk
            query = query.filter(id=[content_type_id, ident[1]])

        if from_date is not None:
            query = query.filter(ostatnio_zmieniony__gte=from_date)

        uczelnia = Uczelnia.objects.get_default()
        if uczelnia:
            ukryte_statusy = uczelnia.ukryte_statusy("api")
            if ukryte_statusy:
                query = query.exclude(status_korekty_id__in=ukryte_statusy)

        for row in (
            query.only(
                "ostatnio_zmieniony",
                "tytul_oryginalny",
                "tytul",
                "jezyk__nazwa",
                "rok",
                "wydawnictwo",
                "zrodlo",
                "informacje",
                "szczegoly",
                "opis_bibliograficzny_autorzy_cache",
                "charakter_formalny__nazwa_w_primo",
                "www",
            )
            .select_related("charakter_formalny", "jezyk")
            .prefetch_related("zrodlo", "slowa_kluczowe")[offset : offset + batch_size]
        ):
            yield {
                "id": get_dc_ident(row.content_type.model, row.object_id),
                "deleted": False,
                "modified": make_naive(
                    row.ostatnio_zmieniony, row.ostatnio_zmieniony.tzinfo
                ),
                "metadata": CacheMetadata(row),
                "sets": ["1"],
            }


class OAIView(View):
    def get(self, request, *args, **kwargs):

        base_url = request.build_absolute_uri(reverse("bpp:oai"))

        url = request.get_full_path()[len(base_url) :]

        if url.startswith("/"):
            url = url[1:]
        if url.endswith("/"):
            url = url[:-1]

        urlparts = url.split("/")

        if len(urlparts) == 0:
            return HttpResponseServerError(
                "500 Internal Server Error",
                "No server was selected, please append server name to url.",
            )

        url = "/".join(urlparts)

        db = BPPOAIDatabase(
            Rekord.objects.all().exclude(charakter_formalny__nazwa_w_primo="")
        )
        oai_server = OAIServerFactory(db, FeedConfig("bpp", base_url))
        return HttpResponse(
            content=oai_server.handleRequest(request.GET),
            content_type="application/xml",
        )



from django.test import TestCase

from bpp.management.commands.look_for_unused_fields import (
    Command as LookForUnusedFields,
)


class TestManagement(TestCase):
    def test_LookForUnusedFields(self):
        LookForUnusedFields().handle(silent=True)



from bpp import const
from bpp.models import mnoznik_dla_monografii
from bpp.models.sloty.wydawnictwo_ciagle import (
    SlotKalkulator_Wydawnictwo_Ciagle_Prog1,
    SlotKalkulator_Wydawnictwo_Ciagle_Prog2,
    SlotKalkulator_Wydawnictwo_Ciagle_Prog3,
)


class SKWZMixin:
    def __init__(self, original, tryb_kalkulacji):
        self.original = original
        self.tryb_kalkulacji = tryb_kalkulacji

    def mnoznik(self, dyscyplina):
        if self.tryb_kalkulacji is not None:
            kod_dziedziny = dyscyplina.kod_dziedziny()
            if kod_dziedziny is not None:
                return mnoznik_dla_monografii(
                    const.DZIEDZINA(kod_dziedziny),
                    self.tryb_kalkulacji,
                    self.original.punkty_kbn,
                )

        return 1


class SlotKalkulator_Wydawnictwo_Zwarte_Prog3(
    SKWZMixin, SlotKalkulator_Wydawnictwo_Ciagle_Prog3
):
    """
    Wydawnictwo zwarte - próg trzeci, ostatni
    Monografia - wydawnictwo spoza wykazu wydawców

    PK 20 + książka autorstwo,
    PK 5 + książka redakcja,
    PK 5 + rozdział.
    """


class SlotKalkulator_Wydawnictwo_Zwarte_Prog2(
    SKWZMixin, SlotKalkulator_Wydawnictwo_Ciagle_Prog2
):
    """
    Wydawnictwo zwarte - próg drugi,
    Monografia - wydawnictwo poziom 1,

    PK 80 + ksiązka autorstwo,
    PK 20 + książka redakcja,
    Pk 20 + rozdział.
    """


class SlotKalkulator_Wydawnictwo_Zwarte_Prog1(
    SKWZMixin, SlotKalkulator_Wydawnictwo_Ciagle_Prog1
):
    """
    Wydawnictwo zwarte - próg pierwszy
    Monogafia - wydawnictwo poziom 2,

    PK 200 + książka autorstwo,
    PK 100 + ksiażka redkacja,
    PK 50 + rozdział.
    """



import codecs


def custom_encode(text: str) -> bytes:
    return text.replace("\x81", "").encode("cp1250")


def custom_decode(binary: bytes, mode: str) -> str:
    try:
        res = codecs.decode(binary, "cp1250")
        return res, len(res)
    except:
        new_bytes = "".join([chr(x) for x in binary if x != 129])
        res = codecs.decode(bytes(new_bytes, "ascii"), "cp1250")
        return res, len(res)

    raise Exception("EDOOFUS")


def custom_search_function(encoding_name):
    return codecs.CodecInfo(custom_encode, custom_decode, name='my_cp1250')


codecs.register(custom_search_function)



# Generated by Django 3.0.7 on 2020-07-27 20:28

import django.db.models.deletion
from django.db import migrations, models

import bpp.const


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("bpp", "0215_auto_20200727_1407"),
    ]

    operations = [
        migrations.CreateModel(
            name="Element_Repozytorium",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("object_id", models.PositiveIntegerField()),
                ("rodzaj", models.CharField(max_length=200)),
                ("nazwa_pliku", models.CharField(max_length=200)),
                (
                    "tryb_dostepu",
                    models.PositiveSmallIntegerField(
                        choices=[
                            (bpp.const.TRYB_DOSTEPU["NIEJAWNY"], "niejawny"),
                            (
                                bpp.const.TRYB_DOSTEPU["TYLKO_W_SIECI"],
                                "tylko w sieci",
                            ),
                            (bpp.const.TRYB_DOSTEPU["JAWNY"], "jawny"),
                        ]
                    ),
                ),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
        ),
    ]



import pytest
from django.utils import timezone

from long_running.exceptions import (
    ObjectDoesNotExistException,
    ProcessingAlreadyStartedException,
)
from long_running.tasks import perform_generic_long_running_task


def test_perform_generic_long_running_task_no_tries_max_tries(wydawnictwo_ciagle):

    with pytest.raises(
        ObjectDoesNotExistException,
    ):
        perform_generic_long_running_task.delay(
            "bpp", "wydawnictwo_ciagle", wydawnictwo_ciagle.pk, no_tries=20, max_tries=1
        )


@pytest.mark.django_db
def test_perform_generic_long_running_task_DoesNotExist(mocker):
    perform_generic_long_running_task.apply_async = mocker.MagicMock()
    perform_generic_long_running_task("test_bpp", "testobjectthatdoesnotexist", 1)
    perform_generic_long_running_task.apply_async.assert_called_once()


def test_perform_generic_long_running_task(operation):
    operation.started_on = timezone.now()
    operation.save()

    with pytest.raises(ProcessingAlreadyStartedException):
        perform_generic_long_running_task("test_bpp", "testoperation", operation.pk)

    operation.started_on = None
    operation.save()

    with pytest.raises(
        NotImplementedError,
    ):
        perform_generic_long_running_task("test_bpp", "testoperation", operation.pk)


def test_Operation_redirect_prefix(operation):
    operation.redirect_prefix = "foo"
    assert operation.get_redirect_prefix() == "foo"


def test_Operation_on_finished_with_error(operation, mocker):
    operation.send_notification = mocker.MagicMock()
    operation.on_finished_with_error()
    operation.send_notification.assert_called_once()



import random
import time
import warnings
from builtins import NotImplementedError
from pprint import pprint
from urllib.parse import parse_qs, quote, urlparse

import requests
from django.db import transaction
from requests import ConnectionError
from requests.exceptions import SSLError
from simplejson.errors import JSONDecodeError

from pbn_api.adapters.wydawnictwo import WydawnictwoPBNAdapter
from pbn_api.const import (
    DEFAULT_BASE_URL,
    NEEDS_PBN_AUTH_MSG,
    PBN_DELETE_PUBLICATION_STATEMENT,
    PBN_GET_INSTITUTION_STATEMENTS,
    PBN_GET_JOURNAL_BY_ID,
    PBN_GET_LANGUAGES_URL,
    PBN_GET_PUBLICATION_BY_ID_URL,
    PBN_POST_PUBLICATIONS_URL,
    PBN_SEARCH_PUBLICATIONS_URL,
)
from pbn_api.exceptions import (
    AccessDeniedException,
    AuthenticationResponseError,
    HttpException,
    NeedsPBNAuthorisationException,
    PraceSerwisoweException,
    SameDataUploadedRecently,
)
from pbn_api.models.sentdata import SentData

from django.contrib.contenttypes.models import ContentType

from django.utils.itercompat import is_iterable


def smart_content(content):
    try:
        return content.decode("utf-8")
    except UnicodeDecodeError:
        return content


class PBNClientTransport:
    def __init__(self, app_id, app_token, base_url, user_token=None):
        self.app_id = app_id
        self.app_token = app_token

        self.base_url = base_url
        if self.base_url is None:
            self.base_url = DEFAULT_BASE_URL

        self.access_token = user_token


class PageableResource:
    def __init__(self, transport, res, url, headers, body=None, method="get"):
        self.url = url
        self.headers = headers
        self.transport = transport
        self.body = body
        self.method = getattr(transport, method)

        try:
            self.page_0 = res["content"]
        except KeyError:
            self.page_0 = []

        self.current_page = res["number"]
        self.total_elements = res["totalElements"]
        self.total_pages = res["totalPages"]
        self.done = False

    def count(self):
        return self.total_elements

    def fetch_page(self, current_page):
        if current_page == 0:
            return self.page_0
        # print(f"FETCH {current_page}")

        kw = {"headers": self.headers}
        if self.body:
            kw["body"] = self.body

        ret = self.method(self.url + f"&page={current_page}", **kw)
        # print(f"FETCH DONE {current_page}")

        try:
            return ret["content"]
        except KeyError:
            return

    def __iter__(self):
        for n in range(0, self.total_pages):
            for elem in self.fetch_page(n):
                yield elem


class OAuthMixin:
    @classmethod
    def get_auth_url(klass, base_url, app_id):
        return f"{base_url}/auth/pbn/api/registration/user/token/{app_id}"

    @classmethod
    def get_user_token(klass, base_url, app_id, app_token, one_time_token):
        headers = {
            "X-App-Id": app_id,
            "X-App-Token": app_token,
        }
        body = {"oneTimeToken": one_time_token}
        url = f"{base_url}/auth/pbn/api/user/token"
        response = requests.post(url=url, json=body, headers=headers)
        try:
            response.json()
        except ValueError:
            raise AuthenticationResponseError(response.content)

        return response.json().get("X-User-Token")

    def authorize(self, base_url, app_id, app_token):
        from pbn_api.conf import settings

        if self.access_token:
            return True

        self.access_token = getattr(settings, "PBN_CLIENT_USER_TOKEN")
        if self.access_token:
            return True

        auth_url = OAuthMixin.get_auth_url(base_url, app_id)

        print(
            f"""I have launched a web browser with {auth_url} ,\nplease log-in,
             then paste the redirected URL below. \n"""
        )
        import webbrowser

        webbrowser.open(auth_url)
        redirect_response = input("Paste the full redirect URL here:")
        one_time_token = parse_qs(urlparse(redirect_response).query).get("ott")[0]
        print("ONE TIME TOKEN", one_time_token)

        self.access_token = OAuthMixin.get_user_token(
            base_url, app_id, app_token, one_time_token
        )

        print("ACCESS TOKEN", self.access_token)
        return True


class RequestsTransport(OAuthMixin, PBNClientTransport):
    def get(self, url, headers=None, fail_on_auth_missing=False):
        sent_headers = {"X-App-Id": self.app_id, "X-App-Token": self.app_token}
        if self.access_token:
            sent_headers["X-User-Token"] = self.access_token

        # Jeżeli ustawimy taki nagłówek dla "niewinnych" zapytań GET, to PBN
        # API odrzuca takie połączenie z kodem 403, stąd nie:
        # if hasattr(self, "access_token"):
        #     sent_headers["X-User-Token"] = self.access_token

        if headers is not None:
            sent_headers.update(headers)

        retries = 0
        MAX_RETRIES = 15

        while retries < MAX_RETRIES:
            try:
                ret = requests.get(self.base_url + url, headers=sent_headers)
                break
            except (SSLError, ConnectionError) as e:
                retries += 1
                time.sleep(random.randint(1, 5))
                if retries >= MAX_RETRIES:
                    raise e

        if ret.status_code == 403:

            if fail_on_auth_missing:
                raise AccessDeniedException(url, smart_content(ret.content))
            # Needs auth
            if ret.json()["message"] == "Access Denied":
                # Autoryzacja użytkownika jest poprawna, jednakże nie ma on po stronie PBN
                # takiego uprawnienia...
                raise AccessDeniedException(url, smart_content(ret.content))

            # elif ret.json['message'] == "Forbidden":  # <== to dostaniemy, gdy token zły lub brak

            if hasattr(self, "authorize"):
                ret = self.authorize(self.base_url, self.app_id, self.app_token)
                if not ret:
                    return

                # Podejmuj ponowną próbę tylko w przypadku udanej autoryzacji
                return self.get(url, headers, fail_on_auth_missing=True)

        if ret.status_code >= 400:
            raise HttpException(ret.status_code, url, smart_content(ret.content))

        try:
            return ret.json()
        except JSONDecodeError as e:
            if ret.status_code == 200 and b"prace serwisowe" in ret.content:
                # open("pbn_client_dump.html", "wb").write(ret.content)
                raise PraceSerwisoweException()
            raise e

    def post(self, url, headers=None, body=None, delete=False):
        if not hasattr(self, "access_token"):
            ret = self.authorize(self.base_url, self.app_id, self.app_token)
            if not ret:
                return
            return self.post(url, headers=headers, body=body, delete=delete)

        sent_headers = {
            "X-App-Id": self.app_id,
            "X-App-Token": self.app_token,
            "X-User-Token": self.access_token,
        }

        if headers is not None:
            sent_headers.update(headers)

        method = requests.post
        if delete:
            method = requests.delete

        ret = method(self.base_url + url, headers=sent_headers, json=body)
        if ret.status_code == 403:

            try:
                ret_json = ret.json()
            except BaseException:
                raise HttpException(
                    ret.status_code,
                    url,
                    "Blad podczas odkodowywania JSON podczas odpowiedzi 403: "
                    + smart_content(ret.content),
                )

            # Needs auth
            if ret_json.get("message") == "Access Denied":
                # Autoryzacja użytkownika jest poprawna, jednakże nie ma on po stronie PBN
                # takiego uprawnienia...
                raise AccessDeniedException(url, smart_content(ret.content))

            if ret_json.get("message") == "Forbidden" and ret_json.get(
                "description"
            ).startswith(NEEDS_PBN_AUTH_MSG):
                # (403, '/api/v1/search/publications?size=10', '{"code":403,"message":"Forbidden",
                # "description":"W celu poprawnej autentykacji należy podać poprawny token użytkownika aplikacji. Podany
                # token użytkownika ... w ramach aplikacji ... nie istnieje lub został
                # unieważniony!"}')
                raise NeedsPBNAuthorisationException(
                    ret.status_code, url, smart_content(ret.content)
                )

            # mpasternak, 5.09.2021: nie do końca jestem pewny, czy kod wywołujący self.authorize (nast. 2 linijki)
            # zostawić w tej sytuacji. Tak było 'historycznie', ale widzę też, że po wywołaniu self.authorize
            # ta funkcja zawsze wykonywała "if ret.status_code >= 403" i zwracała Exception. Teoretycznie
            # to chyba zostało tutaj z powodu klienta command-line, który to na ten moment priorytetem
            # przecież nie jest.
            if hasattr(self, "authorize"):
                self.authorize(self.base_url, self.app_id, self.app_token)
                # self.authorize()

        #
        # mpasternak 7.09.2021, poniżej "przymiarki" do analizowania zwróconych błędów z PBN
        #
        # if ret.status_code == 400:
        #     try:
        #         ret_json = ret.json()
        #     except BaseException:
        #         raise HttpException(
        #             ret.status_code,
        #             url,
        #             "Blad podczas odkodowywania JSON podczas odpowiedzi 400: "
        #             + smart_content(ret.content),
        #         )
        #     if ret_json.get("message") == "Bad Request" and ret_json.get("description") == "Validation failed."
        #     and ret_json.get("details")
        #
        #     HttpException(400, '/api/v1/publications',
        #                   '{"code":400,"message":"Bad Request","description":"Validation failed.",
        #                   "details":{"doi":"DOI jest błędny lub nie udało się pobrać informacji z serwisu DOI!"}}')

        if ret.status_code >= 400:
            raise HttpException(ret.status_code, url, smart_content(ret.content))

        try:
            return ret.json()
        except JSONDecodeError as e:

            if ret.status_code == 200:
                if ret.content == b"":
                    return

                if b"prace serwisowe" in ret.content:
                    # open("pbn_client_dump.html", "wb").write(ret.content)
                    raise PraceSerwisoweException()

            raise e

    def delete(
        self,
        url,
        headers=None,
        body=None,
    ):
        return self.post(url, headers, body, delete=True)

    def _pages(self, method, url, headers=None, body=None, page_size=10, *args, **kw):
        # Stronicowanie zwraca rezultaty w taki sposób:
        # {'content': [{'mongoId': '5e709189878c28a04737dc6f',
        #               'status': 'ACTIVE',
        # ...
        #              'versionHash': '---'}]}],
        #  'first': True,
        #  'last': False,
        #  'number': 0,
        #  'numberOfElements': 10,
        #  'pageable': {'offset': 0,
        #               'pageNumber': 0,
        #               'pageSize': 10,
        #               'paged': True,
        #               'sort': {'sorted': False, 'unsorted': True},
        #               'unpaged': False},
        #  'size': 10,
        #  'sort': {'sorted': False, 'unsorted': True},
        #  'totalElements': 68577,
        #  'totalPages': 6858}

        chr = "?"
        if url.find("?") >= 0:
            chr = "&"

        url = url + f"{chr}size={page_size}"
        chr = "&"

        for elem in kw:
            url += chr + elem + "=" + quote(kw[elem])

        method_function = getattr(self, method)

        if method == "get":
            res = method_function(url, headers)
        elif method == "post":
            res = method_function(url, headers, body=body)
        else:
            raise NotImplementedError

        if "pageable" not in res:
            warnings.warn(
                f"PBNClient.{method}_page request for {url} with headers {headers} did not return a paged resource, "
                f"maybe use PBNClient.{method} (without 'page') instead",
                RuntimeWarning,
            )
            return res
        return PageableResource(
            self, res, url=url, headers=headers, body=body, method=method
        )

    def get_pages(self, url, headers=None, page_size=10, *args, **kw):
        return self._pages(
            "get", url=url, headers=headers, page_size=page_size, *args, **kw
        )

    def post_pages(self, url, headers=None, body=None, page_size=10, *args, **kw):
        # Jak get_pages, ale methoda to post
        if body is None:
            body = kw

        return self._pages(
            "post",
            url=url,
            headers=headers,
            body=body,
            page_size=page_size,
            *args,
            **kw,
        )


class ConferencesMixin:
    def get_conferences(self, *args, **kw):
        return self.transport.get_pages("/api/v1/conferences/page", *args, **kw)

    def get_conferences_mnisw(self, *args, **kw):
        return self.transport.get_pages("/api/v1/conferences/mnisw/page", *args, **kw)

    def get_conference(self, id):
        return self.transport.get(f"/api/v1/conferences/{id}")

    def get_conference_editions(self, id):
        return self.transport.get(f"/api/v1/conferences/{id}/editions")

    def get_conference_metadata(self, id):
        return self.transport.get(f"/api/v1/conferences/{id}/metadata")


class DictionariesMixin:
    def get_countries(self):
        return self.transport.get("/api/v1/dictionary/countries")
        return self.transport.get("/api/v1/dictionary/countries")

    def get_disciplines(self):
        return self.transport.get("/api/v1/dictionary/disciplines")

    def get_languages(self):
        return self.transport.get(PBN_GET_LANGUAGES_URL)


class InstitutionsMixin:
    def get_institutions(self, *args, **kw):
        return self.transport.get_pages("/api/v1/institutions/page", *args, **kw)

    def get_institution_by_id(self, id):
        return self.transport.get_pages(f"/api/v1/institutions/{id}")

    def get_institution_by_version(self, version):
        return self.transport.get_pages(f"/api/v1/institutions/version/{version}")

    def get_institution_metadata(self, id):
        return self.transport.get_pages(f"/api/v1/institutions/{id}/metadata")

    def get_institutions_polon(self):
        return self.transport.get_pages("/api/v1/institutions/polon/page")

    def get_institutions_polon_by_uid(self, uid):
        return self.transport.get(f"/api/v1/institutions/polon/uid/{uid}")

    def get_institutions_polon_by_id(self, id):
        return self.transport.get(f"/api/v1/institutions/polon/{id}")


class InstitutionsProfileMixin:
    # XXX: wymaga autoryzacji
    def get_institution_publications(self, page_size=10):
        return self.transport.get_pages(
            "/api/v1/institutionProfile/publications/page", page_size=page_size
        )

    def get_institution_statements(self, page_size=10):
        return self.transport.get_pages(
            PBN_GET_INSTITUTION_STATEMENTS,
            page_size=page_size,
        )

    def get_institution_statements_of_single_publication(
        self, pbn_uid_id, page_size=50
    ):
        return self.transport.get_pages(
            PBN_GET_INSTITUTION_STATEMENTS + "?publicationId=" + pbn_uid_id,
            page_size=page_size,
        )

    def delete_all_publication_statements(self, publicationId):
        return self.transport.delete(
            f"/api/v1/institutionProfile/publications/{publicationId}",
            body={"all": True, "statementsOfPersons": []},
        )

    def delete_publication_statement(self, publicationId, personId, role):
        return self.transport.delete(
            PBN_DELETE_PUBLICATION_STATEMENT.format(publicationId=publicationId),
            body={"statementsOfPersons": [{"personId": personId, "role": role}]},
        )


class JournalsMixin:
    def get_journals_mnisw(self, *args, **kw):
        return self.transport.get_pages("/api/v1/journals/mnisw/page", *args, **kw)

    def get_journals(self, *args, **kw):
        return self.transport.get_pages("/api/v1/journals/page", *args, **kw)

    def get_journal_by_version(self, version):
        return self.transport.get(f"/api/v1/journals/version/{version}")

    def get_journal_by_id(self, id):
        return self.transport.get(PBN_GET_JOURNAL_BY_ID.format(id=id))

    def get_journal_metadata(self, id):
        return self.transport.get(f"/api/v1/journals/{id}/metadata")


class PersonMixin:
    def get_people_by_institution_id(self, id):
        return self.transport.get(f"/api/v1/person/institution/{id}")

    def get_person_by_natural_id(self, id):
        return self.transport.get(f"/api/v1/person/natural/{id}")

    def get_person_by_orcid(self, orcid):
        return self.transport.get(f"/api/v1/person/orcid/{orcid}")

    def get_people(self, *args, **kw):
        return self.transport.get_pages("/api/v1/person/page", *args, **kw)

    def get_person_by_polon_uid(self, uid):
        return self.transport.get(f"/api/v1/person/polon/{uid}")

    def get_person_by_version(self, version):
        return self.transport.get(f"/api/v1/person/version/{version}")

    def get_person_by_id(self, id):
        return self.transport.get(f"/api/v1/person/{id}")


class PublishersMixin:
    def get_publishers_mnisw(self, *args, **kw):
        return self.transport.get_pages("/api/v1/publishers/mnisw/page", *args, **kw)

    def get_publishers_mnisw_yearlist(self, *args, **kw):
        return self.transport.get_pages(
            "/api/v1/publishers/mnisw/page/yearlist", *args, **kw
        )

    def get_publishers(self, *args, **kw):
        return self.transport.get_pages("/api/v1/publishers/page", *args, **kw)

    def get_publisher_by_version(self, version):
        return self.transport.get(f"/api/v1/publishers/version/{version}")

    def get_publisher_by_id(self, id):
        return self.transport.get(f"/api/v1/publishers/{id}")

    def get_publisher_metadata(self, id):
        return self.transport.get(f"/api/v1/publishers/{id}/metadata")


class PublicationsMixin:
    def get_publication_by_doi(self, doi):
        return self.transport.get(
            f"/api/v1/publications/doi/?doi={quote(doi, safe='')}",
        )

    def get_publication_by_doi_page(self, doi):
        return self.transport.get_pages(
            f"/api/v1/publications/doi/page?doi={quote(doi, safe='')}",
            headers={"doi": doi},
        )

    def get_publication_by_id(self, id):
        return self.transport.get(PBN_GET_PUBLICATION_BY_ID_URL.format(id=id))

    def get_publication_metadata(self, id):
        return self.transport.get(f"/api/v1/publications/id/{id}/metadata")

    def get_publications(self, **kw):
        return self.transport.get_pages("/api/v1/publications/page", **kw)

    def get_publication_by_version(self, version):
        return self.transport.get(f"/api/v1/publications/version/{version}")


class AuthorMixin:
    def get_author_by_id(self, id):
        return self.transport.get(f"/api/v1/author/{id}")


class SearchMixin:
    def search_publications(self, *args, **kw):
        return self.transport.post_pages(PBN_SEARCH_PUBLICATIONS_URL, body=kw)


class PBNClient(
    AuthorMixin,
    ConferencesMixin,
    DictionariesMixin,
    InstitutionsMixin,
    InstitutionsProfileMixin,
    JournalsMixin,
    PersonMixin,
    PublicationsMixin,
    PublishersMixin,
    SearchMixin,
):
    _interactive = False

    def __init__(self, transport: RequestsTransport):
        self.transport = transport

    def post_publication(self, json):
        return self.transport.post(PBN_POST_PUBLICATIONS_URL, body=json)

    def upload_publication(self, rec, force_upload=False, export_pk_zero=None):
        js = WydawnictwoPBNAdapter(rec, export_pk_zero=export_pk_zero).pbn_get_json()
        if not force_upload:
            needed = SentData.objects.check_if_needed(rec, js)
            if not needed:
                raise SameDataUploadedRecently(
                    SentData.objects.get_for_rec(rec).last_updated_on
                )
        try:
            ret = self.post_publication(js)
        except Exception as e:
            SentData.objects.updated(rec, js, uploaded_okay=False, exception=str(e))
            raise e

        return ret, js

    def download_publication(self, doi=None, objectId=None):
        from .integrator import zapisz_mongodb
        from .models import Publication

        assert doi or objectId

        if doi:
            data = self.get_publication_by_doi(doi)
        elif objectId:
            data = self.get_publication_by_id(objectId)

        return zapisz_mongodb(data, Publication)

    @transaction.atomic
    def download_statements_of_publication(self, pub):
        from pbn_api.models import OswiadczenieInstytucji
        from .integrator import pobierz_mongodb, zapisz_oswiadczenie_instytucji

        OswiadczenieInstytucji.objects.filter(publicationId_id=pub.pk).delete()

        pobierz_mongodb(
            self.get_institution_statements_of_single_publication(pub.pk, 5120),
            None,
            fun=zapisz_oswiadczenie_instytucji,
            client=self,
            disable_progress_bar=True,
        )

    def sync_publication(
        self,
        pub,
        force_upload=False,
        delete_statements_before_upload=False,
        export_pk_zero=None,
    ):
        """
        @param delete_statements_before_upload: gdy True, kasuj oświadczenia publikacji przed wysłaniem (jeżeli posiada
        PBN UID)
        """

        # if not pub.doi:
        #     raise WillNotExportError("Ustaw DOI dla publikacji")

        if type(pub) == str:
            # Ciag znaków w postaci wydawnictwo_zwarte:123 pozwoli na podawanie tego
            # parametru do wywołań z linii poleceń
            model, pk = pub.split(":")
            ctype = ContentType.objects.get(app_label="bpp", model=model)
            pub = ctype.model_class().objects.get(pk=pk)

        #
        if (
            delete_statements_before_upload
            and hasattr(pub, "pbn_uid_id")
            and pub.pbn_uid_id is not None
        ):
            try:
                self.delete_all_publication_statements(pub.pbn_uid_id)

                # Jeżeli zostały skasowane dane, to wymuś wysłanie rekordu, niezależnie
                # od stanu tabeli SentData
                force_upload = True
            except HttpException as e:

                NIE_ISTNIEJA = "Nie istnieją oświadczenia dla publikacji"

                ignored_exception = False

                if e.status_code == 400:
                    if e.json:
                        try:
                            msg = e.json["details"]["publicationId"]
                            if NIE_ISTNIEJA in msg:
                                ignored_exception = True
                        except (TypeError, KeyError):
                            pass
                    else:
                        if NIE_ISTNIEJA in e.content:
                            ignored_exception = True

                if not ignored_exception:
                    raise e

        # Wgraj dane do PBN
        ret, js = self.upload_publication(
            pub, force_upload=force_upload, export_pk_zero=export_pk_zero
        )

        # Pobierz zwrotnie dane z PBN
        publication = self.download_publication(objectId=ret["objectId"])
        self.download_statements_of_publication(publication)

        # Utwórz obiekt zapisanych danych. Dopiero w tym miejscu, bo jeżeli zostanie
        # utworzony nowy rekord po stronie PBN, to pbn_uid_id musi wskazywać na
        # bazę w tabeli Publication, która została chwile temu pobrana...
        SentData.objects.updated(pub, js, pbn_uid_id=ret["objectId"])

        if pub.pbn_uid_id != ret["objectId"]:
            pub.pbn_uid = publication
            pub.save()

    def demo(self):
        from bpp.models import Wydawnictwo_Ciagle

        pub = Wydawnictwo_Ciagle.objects.filter(rok=2020).exclude(doi=None).first()
        self.sync_publication(pub, force_upload=True)

    def exec(self, cmd):
        try:
            fun = getattr(self, cmd[0])
        except AttributeError as e:
            if self._interactive:
                print("No such command: %s" % cmd)
                return
            else:
                raise e

        def extract_arguments(lst):
            args = ()
            kw = {}
            for elem in lst:
                if elem.find(":") >= 1:
                    k, n = elem.split(":", 1)
                    kw[k] = n
                else:
                    args += (elem,)

            return args, kw

        args, kw = extract_arguments(cmd[1:])
        res = fun(*args, **kw)
        if type(res) == dict:
            pprint(res)
        elif is_iterable(res):
            if self._interactive and hasattr(res, "total_elements"):
                print(
                    "Incoming data: no_elements=",
                    res.total_elements,
                    "no_pages=",
                    res.total_pages,
                )
                input("Press ENTER to continue> ")
            for elem in res:
                pprint(elem)

    def interactive(self):
        self._interactive = True
        while True:
            cmd = input("cmd> ")
            if cmd == "exit":
                break
            self.exec(cmd.split(" "))



from django.db import models

from .base import BasePBNMongoDBModel

from bpp import const
from bpp.models import LinkDoPBNMixin


class Journal(LinkDoPBNMixin, BasePBNMongoDBModel):
    url_do_pbn = const.LINK_PBN_DO_ZRODLA
    atrybut_dla_url_do_pbn = "pk"

    class Meta:
        verbose_name = "Zródło w PBN API"
        verbose_name_plural = "Zródła w PBN API"

    title = models.TextField(null=True, blank=True, db_index=True)
    websiteLink = models.TextField(null=True, blank=True, db_index=True)
    issn = models.TextField(null=True, blank=True, db_index=True)
    eissn = models.TextField(null=True, blank=True, db_index=True)
    mniswId = models.IntegerField(null=True, blank=True, db_index=True)

    pull_up_on_save = ["title", "websiteLink", "issn", "eissn", "mniswId"]

    def __str__(self):
        return (
            f"{self.title}, ISSN: {self.issn or '-'}, "
            f"EISSN: {self.eissn or '-'}, MNISW ID: {self.mniswId or '-'}"
        )

    def rekord_w_bpp(self):
        from bpp.models import Zrodlo

        try:
            return Zrodlo.objects.filter(pbn_uid_id=self.pk).first()
        except Zrodlo.DoesNotExist:
            return



from django import forms
from django.db.models import JSONField, TextField

from formdefaults.models import (
    FormFieldDefaultValue,
    FormFieldRepresentation,
    FormRepresentation,
)

from django.contrib import admin
from django.contrib.auth import get_user_model

# Register your models here.

WideTextInput = {"widget": forms.TextInput(attrs={"style": "width: 100%"})}


class FormFieldRepresentationInline(admin.TabularInline):
    model = FormFieldRepresentation
    fields = ["name", "label"]
    readonly_fields = ["name", "label"]
    extra = 0
    can_delete = False
    show_change_link = False

    def has_add_permission(self, request, obj):
        return False

    formfield_overrides = {
        TextField: WideTextInput,
    }


class FormFieldDefaultValueForm(forms.ModelForm):
    user = forms.ModelChoiceField(
        queryset=get_user_model().objects.all(), empty_label="każdy", required=False
    )

    class Meta:
        fields = ["user", "field", "value"]
        model = FormFieldDefaultValue


class FormFieldDefaultValueInline(admin.TabularInline):
    model = FormFieldDefaultValue
    form = FormFieldDefaultValueForm
    formfield_overrides = {
        JSONField: WideTextInput,
    }
    extra = 0


@admin.register(FormRepresentation)
class FormRepresentationAdmin(admin.ModelAdmin):
    list_display = ["label", "full_name"]
    inlines = [FormFieldDefaultValueInline]
    readonly_fields = list_display
    fields = ["label", "full_name", "html_before", "html_after"]



# -*- coding: utf-8 -*-


from django.db import migrations, models
from django.db.models import CASCADE


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0054_obiekt_jednostka_pole_aktualna'),
    ]

    operations = [
        migrations.CreateModel(
            name='Jednostka_Wydzial',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('od', models.DateField(null=True, blank=True)),
                ('do', models.DateField(null=True, blank=True)),
                ('jednostka', models.ForeignKey(on_delete=models.CASCADE, to='bpp.Jednostka')),
                ('wydzial', models.ForeignKey(on_delete=models.CASCADE, to='bpp.Wydzial')),
            ],
        ),
    ]



from .fields import PunktacjaWewnetrznaReportType, ReportType, multiseek_fields  # noqa

multiseek_report_types = [
    ReportType("list", "lista"),
    ReportType("table", "tabela"),
    PunktacjaWewnetrznaReportType(
        "pkt_wewn", "punktacja sumaryczna z punktacją wewnętrzna"
    ),
    ReportType("pkt_wewn_bez", "punktacja sumaryczna"),
    ReportType("numer_list", "numerowana lista z uwagami", public=False),
    ReportType("table_cytowania", "tabela z liczbą cytowań", public=False),
    PunktacjaWewnetrznaReportType(
        "pkt_wewn_cytowania",
        "punktacja sumaryczna z punktacją wewnętrzna z liczbą cytowań",
        public=False,
    ),
    ReportType(
        "pkt_wewn_bez_cytowania", "punktacja sumaryczna z liczbą cytowań", public=False
    ),
]



from tee.models import Log
from tee.utils import last_n_lines

from django.contrib import admin

from django.utils.safestring import mark_safe


@admin.register(Log)
class LogAdmin(admin.ModelAdmin):
    list_display = [
        "cmd_name",
        "started_on",
        "finished_on",
        "finished_successfully",
        "last_5_lines",
    ]

    list_per_page = 10

    readonly_fields = [
        "started_on",
        "finished_on",
        "finished_successfully",
        "command_name",
        "args",
        "stdout",
        "stderr",
        "traceback",
    ]

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def cmd_name(self, obj):
        args = ""
        if obj.args:
            args = " {' '.join(obj.args)}"
        return f"{obj.command_name}" + args

    def finished_successfully(self, obj: Log):
        if obj.exit_code == 0:
            return True
        return False

    def last_5_lines(self, obj):
        s = obj.stderr

        if obj.traceback:
            s = obj.traceback

        if not s:
            s = obj.stdout

        r = last_n_lines(s, nlines=5)
        if r is None:
            return
        return mark_safe(f"<pre>{r}</pre>")

    last_5_lines.short_description = "Results"



from django.core.management import BaseCommand

from pbn_api.client import PBNClient, RequestsTransport
from pbn_api.conf import settings

from bpp.models import Uczelnia


class PBNBaseCommand(BaseCommand):
    def add_arguments(self, parser):

        app_id = getattr(settings, "PBN_CLIENT_APP_ID")
        app_token = getattr(settings, "PBN_CLIENT_APP_TOKEN")
        base_url = getattr(settings, "PBN_CLIENT_BASE_URL")
        user_token = None

        uczelnia = Uczelnia.objects.get_default()
        if uczelnia is not None:
            if uczelnia.pbn_app_name:
                app_id = uczelnia.pbn_app_name
            if uczelnia.pbn_app_token:
                app_token = uczelnia.pbn_app_token
            if uczelnia.pbn_api_root:
                base_url = uczelnia.pbn_api_root
            if uczelnia.pbn_api_user_id:
                user_token = uczelnia.pbn_api_user.pbn_token

        parser.add_argument("--app-id", default=app_id)
        parser.add_argument("--app-token", default=app_token)
        parser.add_argument("--base-url", default=base_url)
        parser.add_argument("--user-token", default=user_token)

    def get_client(self, app_id, app_token, base_url, user_token, verbose=False):
        transport = RequestsTransport(app_id, app_token, base_url, user_token)
        if verbose:
            print("App ID\t\t", app_id)
            print("App token\t", app_token)
            print("Base URL\t", base_url)
            print("User token\t", user_token)
        return PBNClient(transport)



from decimal import Decimal

import pytest

from django.contrib.contenttypes.models import ContentType

from bpp.const import TO_AUTOR, TO_REDAKTOR
from bpp.models import (
    Autor_Dyscyplina,
    Cache_Punktacja_Autora,
    Cache_Punktacja_Dyscypliny,
    Charakter_Formalny,
    Dyscyplina_Naukowa,
    Rekord,
    Typ_KBN,
    Typ_Odpowiedzialnosci,
    Uczelnia,
)
from bpp.models.sloty.core import IPunktacjaCacher, ISlot
from bpp.models.sloty.exceptions import CannotAdapt
from bpp.models.sloty.wydawnictwo_ciagle import (
    SlotKalkulator_Wydawnictwo_Ciagle_Prog2,
    SlotKalkulator_Wydawnictwo_Ciagle_Prog3,
)
from bpp.models.sloty.wydawnictwo_zwarte import (
    SlotKalkulator_Wydawnictwo_Zwarte_Prog1,
    SlotKalkulator_Wydawnictwo_Zwarte_Prog2,
    SlotKalkulator_Wydawnictwo_Zwarte_Prog3,
)


@pytest.mark.parametrize(
    "rekord,ustaw_rok,punkty_kbn",
    [
        # (pytest.lazy_fixture("wydawnictwo_zwarte"), 2017, 20),
        (pytest.lazy_fixture("wydawnictwo_ciagle"), 2017, 30)
    ],
)
@pytest.mark.django_db
def test_slot_wszyscy_slot_wszystkie_dyscypliny(
    rekord,
    ustaw_rok,
    punkty_kbn,
    autor_jan_kowalski,
    autor_jan_nowak,
    dyscyplina1,
    dyscyplina2,
    jednostka,
    typy_odpowiedzialnosci,
):
    rekord.rok = ustaw_rok
    rekord.punkty_kbn = punkty_kbn
    rekord.save()

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_nowak,
        dyscyplina_naukowa=dyscyplina1,
        rok=ustaw_rok,
    )

    rekord.dodaj_autora(autor_jan_nowak, jednostka, dyscyplina_naukowa=dyscyplina1)

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina2,
        rok=ustaw_rok,
    )
    rekord.dodaj_autora(autor_jan_kowalski, jednostka, dyscyplina_naukowa=dyscyplina2)

    slot = ISlot(rekord)

    assert len(slot.dyscypliny) == 2
    assert slot.wszyscy() == 2


@pytest.mark.django_db
def test_autorzy_z_dyscypliny(
    ciagle_z_dyscyplinami,
    autor_jan_nowak,
    autor_jan_kowalski,
    dyscyplina1,
    dyscyplina2,
    dyscyplina3,
):
    ciagle_z_dyscyplinami.punkty_kbn = 30
    ciagle_z_dyscyplinami.rok = 2017
    ciagle_z_dyscyplinami.save()

    slot = ISlot(ciagle_z_dyscyplinami)

    assert len(slot.autorzy_z_dyscypliny(dyscyplina1)) == 1
    assert len(slot.autorzy_z_dyscypliny(dyscyplina2)) == 1
    assert len(slot.autorzy_z_dyscypliny(dyscyplina3)) == 0

    assert len(slot.autorzy_z_dyscypliny(dyscyplina1, TO_AUTOR)) == 1
    assert len(slot.autorzy_z_dyscypliny(dyscyplina2, TO_AUTOR)) == 1
    assert len(slot.autorzy_z_dyscypliny(dyscyplina3, TO_AUTOR)) == 0

    assert len(slot.autorzy_z_dyscypliny(dyscyplina1, TO_REDAKTOR)) == 0
    assert len(slot.autorzy_z_dyscypliny(dyscyplina2, TO_REDAKTOR)) == 0
    assert len(slot.autorzy_z_dyscypliny(dyscyplina3, TO_REDAKTOR)) == 0

    # Sprawdź, czy 'autorzy_z_dyscypliny' zwraca tylko afiliowanych (#927)
    # Pierwszy autor to Jan Nowak z dyscyplina "memetyka stosowana" czyli dyscyplina1
    # Wcześniej przypisz tego autora do tej dyscypliny na ten rok

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_nowak, rok=2017, dyscyplina_naukowa=dyscyplina1
    )

    wca1 = ciagle_z_dyscyplinami.autorzy_set.first()

    wca1.afiliuje = True
    wca1.save()
    assert len(slot.autorzy_z_dyscypliny(dyscyplina1)) == 1

    wca1.afiliuje = False
    wca1.save()
    assert len(slot.autorzy_z_dyscypliny(dyscyplina1)) == 0


@pytest.mark.django_db
@pytest.mark.parametrize(
    "ustaw_rok,punkty,ma_byc_1,ma_byc_2",
    [
        (2017, 30, "30.0000", "1.0000"),
        (2017, 20, "14.1421", "0.7071"),
        (2017, 25, "17.6777", "0.7071"),
        (2017, 15, "7.5000", "0.5000"),
        (2017, 5, "2.5000", "0.5000"),
        (2019, 200, "200.0000", "1.0000"),
        (2019, 140, "140.0000", "1.0000"),
        (2019, 100, "100.0000", "1.0000"),
        (2019, 70, "49.4975", "0.7071"),
        (2019, 40, "28.2843", "0.7071"),
        (2019, 20, "10.0000", "0.5000"),
        (2019, 5, "2.5000", "0.5000"),
    ],
)
def test_slot_artykuly(
    ciagle_z_dyscyplinami,
    autor_jan_nowak,
    autor_jan_kowalski,
    dyscyplina1,
    dyscyplina2,
    dyscyplina3,
    ustaw_rok,
    punkty,
    ma_byc_1,
    ma_byc_2,
):
    ciagle_z_dyscyplinami.punkty_kbn = punkty
    ciagle_z_dyscyplinami.rok = ustaw_rok
    ciagle_z_dyscyplinami.save()

    slot = ISlot(ciagle_z_dyscyplinami)

    assert f"{slot.punkty_pkd(dyscyplina1):.4f}" == ma_byc_1
    assert f"{slot.punkty_pkd(dyscyplina2):.4f}" == ma_byc_1
    assert slot.punkty_pkd(dyscyplina3) is None

    assert f"{slot.pkd_dla_autora(autor_jan_kowalski):.4f}" == ma_byc_1
    assert f"{slot.pkd_dla_autora(autor_jan_nowak):.4f}" == ma_byc_1

    assert f"{slot.slot_dla_autora(autor_jan_kowalski):.4f}" == ma_byc_2
    assert f"{slot.slot_dla_autora(autor_jan_nowak):.4f}" == ma_byc_2
    assert slot.slot_dla_autora_z_dyscypliny(dyscyplina3) is None

    assert f"{slot.slot_dla_dyscypliny(dyscyplina1):.4f}" == ma_byc_2
    assert f"{slot.slot_dla_dyscypliny(dyscyplina2):.4f}" == ma_byc_2
    assert slot.slot_dla_dyscypliny(dyscyplina3) is None


@pytest.mark.django_db
def test_IPunktacjaCacher(
    ciagle_z_dyscyplinami,
    denorms,
    autor_jan_nowak,
    autor_jan_kowalski,
    dyscyplina1,
    dyscyplina2,
    dyscyplina3,
):
    ciagle_z_dyscyplinami.punkty_kbn = 30
    ciagle_z_dyscyplinami.rok = 2017
    ciagle_z_dyscyplinami.save()
    denorms.flush()

    ipc = IPunktacjaCacher(ciagle_z_dyscyplinami)
    assert ipc.canAdapt()

    ipc.removeEntries()
    ipc.rebuildEntries()

    assert Cache_Punktacja_Dyscypliny.objects.count() == 2
    assert Cache_Punktacja_Autora.objects.count() == 2


@pytest.mark.django_db
def test_IPunktacjaCacher_brak_afiliacji(
    ciagle_z_dyscyplinami,
    autor_jan_nowak,
    autor_jan_kowalski,
    dyscyplina1,
    dyscyplina2,
    dyscyplina3,
):
    ciagle_z_dyscyplinami.punkty_kbn = 30
    ciagle_z_dyscyplinami.rok = 2017
    ciagle_z_dyscyplinami.save()

    ciagle_z_dyscyplinami.autorzy_set.update(afiliuje=False)

    ipc = IPunktacjaCacher(ciagle_z_dyscyplinami)
    assert ipc.canAdapt()
    ipc.removeEntries()
    ipc.rebuildEntries()

    assert Cache_Punktacja_Dyscypliny.objects.count() == 0
    assert Cache_Punktacja_Autora.objects.count() == 0


@pytest.mark.django_db
def test_slotkalkulator_wydawnictwo_ciagle_prog3_punkty_pkd(
    ciagle_z_dyscyplinami, dyscyplina1
):
    ciagle_z_dyscyplinami.rok = 2018
    ciagle_z_dyscyplinami.punkty_kbn = 5
    ciagle_z_dyscyplinami.save()

    slot = SlotKalkulator_Wydawnictwo_Ciagle_Prog3(ciagle_z_dyscyplinami)

    assert slot.punkty_pkd(dyscyplina1) == 2.5
    assert slot.slot_dla_autora_z_dyscypliny(dyscyplina1) == 0.5
    assert slot.slot_dla_dyscypliny(dyscyplina1) == 0.5

    # Zdejmij afiliacje i sprawdz czy k_przez_m wyjdzie zero
    for aut in ciagle_z_dyscyplinami.autorzy_set.all():
        aut.afiliuje = False
        Autor_Dyscyplina.objects.get_or_create(
            autor=aut.autor,
            dyscyplina_naukowa=aut.dyscyplina_naukowa,
            rok=ciagle_z_dyscyplinami.rok,
        )
        aut.save()

    slot = SlotKalkulator_Wydawnictwo_Ciagle_Prog2(ciagle_z_dyscyplinami)
    assert str(round(slot.slot_dla_dyscypliny(dyscyplina1), 4)) == "0.0000"


@pytest.mark.django_db
def test_slotkalkulator_wydawnictwo_ciagle_prog2_punkty_pkd(
    ciagle_z_dyscyplinami, dyscyplina1
):
    ciagle_z_dyscyplinami.rok = 2018
    ciagle_z_dyscyplinami.punkty_kbn = 20
    ciagle_z_dyscyplinami.save()

    slot = SlotKalkulator_Wydawnictwo_Ciagle_Prog2(ciagle_z_dyscyplinami)

    assert str(round(slot.punkty_pkd(dyscyplina1), 4)) == "14.1421"
    assert str(round(slot.slot_dla_autora_z_dyscypliny(dyscyplina1), 4)) == "0.7071"
    assert str(round(slot.slot_dla_dyscypliny(dyscyplina1), 4)) == "0.7071"

    assert isinstance(slot.pierwiastek_k_przez_m(dyscyplina1), Decimal)

    # Zdejmij afiliacje i sprawdz czy k_przez_m wyjdzie zero
    for aut in ciagle_z_dyscyplinami.autorzy_set.all():
        aut.afiliuje = False
        Autor_Dyscyplina.objects.get_or_create(
            autor=aut.autor,
            dyscyplina_naukowa=aut.dyscyplina_naukowa,
            rok=ciagle_z_dyscyplinami.rok,
        )
        aut.save()

    slot = SlotKalkulator_Wydawnictwo_Ciagle_Prog2(ciagle_z_dyscyplinami)
    assert str(round(slot.slot_dla_dyscypliny(dyscyplina1), 4)) == "0.0000"


@pytest.mark.django_db
def test_ISlot_wydawnictwo_zwarte_zakres_lat_nie_ten(zwarte_z_dyscyplinami):
    zwarte_z_dyscyplinami.rok = 2016

    with pytest.raises(CannotAdapt):
        ISlot(zwarte_z_dyscyplinami)

    zwarte_z_dyscyplinami.rok = 2021
    ISlot(zwarte_z_dyscyplinami)

    zwarte_z_dyscyplinami.rok = 2020
    ISlot(zwarte_z_dyscyplinami)

    zwarte_z_dyscyplinami.rok = 2023
    ISlot(zwarte_z_dyscyplinami)

    zwarte_z_dyscyplinami.rok = 2024
    with pytest.raises(CannotAdapt):
        ISlot(zwarte_z_dyscyplinami)


@pytest.mark.django_db
def test_ISlot_wydawnictwo_zwarte_redakcja_i_autorstwo(zwarte_z_dyscyplinami):
    a1 = zwarte_z_dyscyplinami.autorzy_set.first()
    a1.typ_odpowiedzialnosci = Typ_Odpowiedzialnosci.objects.get(skrot="red.")
    a1.save()

    zwarte_z_dyscyplinami.rok = 2021
    zwarte_z_dyscyplinami.save()

    with pytest.raises(CannotAdapt, match="ma jednocześnie"):
        ISlot(zwarte_z_dyscyplinami)


@pytest.mark.django_db
def test_ISlot_wydawnictwo_zwarte_bez_red_bez_aut(zwarte_z_dyscyplinami):
    zwarte_z_dyscyplinami.autorzy_set.all().delete()
    zwarte_z_dyscyplinami.rok = 2021
    zwarte_z_dyscyplinami.save()

    with pytest.raises(CannotAdapt, match="nie posiada"):
        ISlot(zwarte_z_dyscyplinami)


@pytest.mark.django_db
def test_ISlot_wydawnictwo_Zwarte_nie_te_punkty(zwarte_z_dyscyplinami):
    zwarte_z_dyscyplinami.punkty_kbn = 12345
    zwarte_z_dyscyplinami.rok = 2021
    with pytest.raises(CannotAdapt, match="nie można dopasować do żadnej z grup"):
        ISlot(zwarte_z_dyscyplinami)


@pytest.mark.django_db
def test_ISlot_wydawnictwo_zwarte_tier3(zwarte_z_dyscyplinami):
    zwarte_z_dyscyplinami.rok = 2021
    i = ISlot(zwarte_z_dyscyplinami)
    assert isinstance(i, SlotKalkulator_Wydawnictwo_Zwarte_Prog3)


@pytest.mark.django_db
def test_ISlot_wydawnictwo_zwarte_tier3_rok_2022(zwarte_z_dyscyplinami):
    zwarte_z_dyscyplinami.rok = 2022
    i = ISlot(zwarte_z_dyscyplinami)
    assert isinstance(i, SlotKalkulator_Wydawnictwo_Zwarte_Prog3)


@pytest.mark.django_db
def test_ISlot_wydawnictwo_zwarte_tier2(zwarte_z_dyscyplinami, wydawca):
    rok = 2021
    wydawca.poziom_wydawcy_set.create(rok=rok, poziom=1)
    zwarte_z_dyscyplinami.punkty_kbn = 80
    zwarte_z_dyscyplinami.rok = rok
    i = ISlot(zwarte_z_dyscyplinami)
    assert isinstance(i, SlotKalkulator_Wydawnictwo_Zwarte_Prog2)


@pytest.mark.django_db
def test_ISlot_wydawnictwo_zwarte_tier1(zwarte_z_dyscyplinami, wydawca):
    rok = 2021
    wydawca.poziom_wydawcy_set.create(rok=rok, poziom=2)
    zwarte_z_dyscyplinami.punkty_kbn = 200
    zwarte_z_dyscyplinami.rok = rok
    i = ISlot(zwarte_z_dyscyplinami)
    assert isinstance(i, SlotKalkulator_Wydawnictwo_Zwarte_Prog1)


@pytest.mark.django_db
def test_ISlot_wydawnictwo_zwarte_tier1_brak_afiliacji(
    zwarte_z_dyscyplinami, wydawca, rok, dyscyplina1, dyscyplina2
):
    rok = 2021
    wydawca.poziom_wydawcy_set.create(rok=rok, poziom=2)
    zwarte_z_dyscyplinami.punkty_kbn = 200
    zwarte_z_dyscyplinami.rok = rok

    i = ISlot(zwarte_z_dyscyplinami)
    assert i.autorzy_z_dyscypliny(dyscyplina1)

    zwarte_z_dyscyplinami.autorzy_set.update(afiliuje=False)
    assert not i.autorzy_z_dyscypliny(dyscyplina1)


@pytest.mark.django_db
def test_ISlot_wydawnictwo_ciagle_bez_punktow_kbn(ciagle_z_dyscyplinami):
    ciagle_z_dyscyplinami.punkty_kbn = 0

    with pytest.raises(CannotAdapt, match="nie pozwalają"):
        ISlot(ciagle_z_dyscyplinami)


@pytest.mark.django_db
def test_cache_slotow_kasowanie_wpisow_przy_zmianie_pk_ciagle(
    ciagle_z_dyscyplinami, denorms
):
    ciagle_z_dyscyplinami.punkty_kbn = 30
    ciagle_z_dyscyplinami.rok = 2017
    ciagle_z_dyscyplinami.save()

    assert ISlot(ciagle_z_dyscyplinami) is not None

    denorms.flush()

    ctype = ContentType.objects.get_for_model(ciagle_z_dyscyplinami).pk
    assert (
        Cache_Punktacja_Autora.objects.filter(
            rekord_id=[ctype, ciagle_z_dyscyplinami.pk]
        ).count()
        == 2
    )

    ciagle_z_dyscyplinami.punkty_kbn = 0
    ciagle_z_dyscyplinami.save()

    denorms.flush()
    assert (
        Cache_Punktacja_Autora.objects.filter(
            rekord_id=[ctype, ciagle_z_dyscyplinami.pk]
        ).count()
        == 0
    )


@pytest.mark.django_db
def test_cache_slotow_kasowanie_wpisow_przy_zmianie_pk_zwarte(
    zwarte_z_dyscyplinami, denorms
):
    zwarte_z_dyscyplinami.punkty_kbn = 20
    zwarte_z_dyscyplinami.rok = 2017
    zwarte_z_dyscyplinami.save()

    assert ISlot(zwarte_z_dyscyplinami) is not None

    denorms.flush()

    ctype = ContentType.objects.get_for_model(zwarte_z_dyscyplinami).pk
    assert (
        Cache_Punktacja_Autora.objects.filter(
            rekord_id=[ctype, zwarte_z_dyscyplinami.pk]
        ).count()
        == 2
    )

    zwarte_z_dyscyplinami.punkty_kbn = 0
    zwarte_z_dyscyplinami.save()

    denorms.flush()

    assert (
        Cache_Punktacja_Autora.objects.filter(
            rekord_id=[ctype, zwarte_z_dyscyplinami.pk]
        ).count()
        == 0
    )


@pytest.mark.django_db
def test_sloty_prace_wieloosrodkowe(zwarte_z_dyscyplinami, typy_kbn):
    zwarte_z_dyscyplinami.typ_kbn = Typ_KBN.objects.get(skrot="PW")
    zwarte_z_dyscyplinami.save()

    with pytest.raises(CannotAdapt, match="dla prac wielo"):
        ISlot(zwarte_z_dyscyplinami)


@pytest.mark.django_db
def test_ISlot_patent(patent):
    with pytest.raises(CannotAdapt):
        ISlot(patent)


@pytest.mark.django_db
def test_ISlot_mnozniki_dla_dyscyplin_z_dziedziony_np_humanistycznych_zwarte(
    wydawnictwo_zwarte,
    autor_jan_kowalski,
    jednostka,
    typy_kbn,
    charaktery_formalne,
    wydawca,
    denorms,
):
    d_teologia = Dyscyplina_Naukowa.objects.create(
        kod="07.001", nazwa="Teologia stosowana"
    )

    ROK = 2019

    wydawca.poziom_wydawcy_set.create(rok=ROK, poziom=2)

    wydawnictwo_zwarte.punkty_kbn = 200.0
    wydawnictwo_zwarte.rok = ROK
    wydawnictwo_zwarte.wydawca = wydawca
    wydawnictwo_zwarte.charakter_formalny = Charakter_Formalny.objects.get(skrot="KSP")
    wydawnictwo_zwarte.save()

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski, rok=ROK, dyscyplina_naukowa=d_teologia
    )

    wydawnictwo_zwarte.dodaj_autora(
        autor_jan_kowalski, jednostka, dyscyplina_naukowa=d_teologia
    )

    ISlot(wydawnictwo_zwarte)
    denorms.flush()

    assert Cache_Punktacja_Autora.objects.first().pkdaut == 300


@pytest.mark.django_db
@pytest.mark.parametrize("punktacja,oczekiwana", [(30, 30), (20, 20), (0.5, 0.5)])
def test_ISlot_mnozniki_dla_dyscyplin_z_dziedziony_np_humanistycznych_ciagle(
    wydawnictwo_ciagle,
    autor_jan_kowalski,
    jednostka,
    typy_kbn,
    charaktery_formalne,
    wydawca,
    punktacja,
    oczekiwana,
    denorms,
):
    d_teologia = Dyscyplina_Naukowa.objects.create(
        kod="07.001", nazwa="Teologia stosowana"
    )

    ROK = 2018

    wydawca.poziom_wydawcy_set.create(rok=ROK, poziom=2)

    wydawnictwo_ciagle.punkty_kbn = Decimal(punktacja)
    wydawnictwo_ciagle.rok = ROK
    wydawnictwo_ciagle.charakter_formalny = Charakter_Formalny.objects.get(skrot="AC")
    wydawnictwo_ciagle.save()

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski, dyscyplina_naukowa=d_teologia, rok=ROK
    )

    wydawnictwo_ciagle.dodaj_autora(
        autor_jan_kowalski, jednostka, dyscyplina_naukowa=d_teologia
    )

    wydawnictwo_ciagle.refresh_from_db()

    ISlot(wydawnictwo_ciagle)
    denorms.flush()

    assert Cache_Punktacja_Autora.objects.first().pkdaut == oczekiwana


@pytest.mark.parametrize("akcja", ["wszystko", None])
@pytest.mark.django_db
def test_autor_Autor_zbieraj_sloty(zwarte_z_dyscyplinami, akcja, denorms):
    zwarte_z_dyscyplinami.punkty_kbn = 20
    zwarte_z_dyscyplinami.rok = 2017
    zwarte_z_dyscyplinami.save()

    denorms.flush()

    a = zwarte_z_dyscyplinami.autorzy_set.first().autor
    res = a.zbieraj_sloty(
        1, zwarte_z_dyscyplinami.rok, zwarte_z_dyscyplinami.rok, akcja=akcja
    )
    assert res == (
        10.0,
        [
            Rekord.objects.get_for_model(zwarte_z_dyscyplinami)
            .cache_punktacja_autora_query_set.first()
            .pk
        ],
        0.5,
    )


@pytest.mark.django_db
def test_ISlot_ukryty_status_nie_licz_punktow(
    zwarte_z_dyscyplinami, przed_korekta, po_korekcie, uczelnia
):
    zwarte_z_dyscyplinami.punkty_kbn = 20
    zwarte_z_dyscyplinami.rok = 2017

    zwarte_z_dyscyplinami.status_korekty = przed_korekta
    zwarte_z_dyscyplinami.save()

    Uczelnia.objects.get_default().ukryj_status_korekty_set.create(
        status_korekty=przed_korekta
    )

    with pytest.raises(CannotAdapt):
        ISlot(zwarte_z_dyscyplinami)

    zwarte_z_dyscyplinami.status_korekty = po_korekcie
    zwarte_z_dyscyplinami.save()

    ISlot(zwarte_z_dyscyplinami)



import pytest
from model_bakery import baker

from bpp.models import Autor, Praca_Doktorska, Praca_Habilitacyjna, Tytul


@pytest.mark.django_db
@pytest.mark.parametrize("klass", [Praca_Doktorska, Praca_Habilitacyjna])
def test_autor_bez_tytulu(klass, typy_odpowiedzialnosci):
    "Upewnij się, że tytuł autora nie wchodzi do opisu pracy doktorskiej"
    t = baker.make(Tytul, nazwa="profesor doktor", skrot="prof. dr")
    a = baker.make(Autor, imiona="Test", nazwisko="Foo", tytul=t)
    p = baker.make(klass, autor=a)
    assert "prof. dr" not in p.opis_bibliograficzny().lower()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-09 09:29
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import django_fsm


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0121_rekord_ostatnio_zmieniony_dla'),
        ('import_dyscyplin', '0002_import_dyscyplin_web_page_uid'),
    ]

    operations = [
        migrations.CreateModel(
            name='Import_Dyscyplin_Row',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('row_no', models.IntegerField()),
                ('original', django.contrib.postgres.fields.jsonb.JSONField()),
                ('autor_id', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Autor')),
                ('jednostka_id', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Jednostka')),
            ],
        ),
        migrations.AlterField(
            model_name='import_dyscyplin',
            name='stan',
            field=django_fsm.FSMField(choices=[('nowy', 'nowy'), ('przeanalizowany', 'przeanalizowany'), ('błędny', 'błędny'), ('zintegrowany', 'zintegrowany')], default='nowy', max_length=50, protected=True),
        ),
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='parent',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='import_dyscyplin.Import_Dyscyplin'),
        ),
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='wydzial_id',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Wydzial'),
        ),
    ]



# Generated by Django 3.2.15 on 2022-09-10 14:46

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0327_uczelnia_pokazuj_formularz_zglaszania_publikacji"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("zglos_publikacje", "0010_auto_20220818_0012"),
    ]

    operations = [
        migrations.AlterField(
            model_name="zgloszenie_publikacji",
            name="rodzaj_zglaszanej_publikacji",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (1, "artykuł naukowy lub monografia"),
                    (2, "pozostałe rodzaje"),
                ],
                help_text="Dla artykułów naukowych i monografii może być wymagane wprowadzenie informacji o opłatach za publikację w ostatnim etapie wypełniania formularza. ",
                verbose_name="Rodzaj zgłaszanej publikacji",
            ),
        ),
        migrations.AlterField(
            model_name="zgloszenie_publikacji",
            name="strona_www",
            field=models.URLField(
                blank=True,
                help_text="Pole opcjonalne. Adres URL lokalizacji pełnego tekstu pracy (dostęp otwarty lub nie). Jeżeli praca posiada numer DOI, wpisz go w postaci adresu URL czyli https://dx.doi.org/[NUMER_DOI]. Jeżeli praca nie posiada numeru DOI bądź nie jest dostępna w sieci, pozostaw to pole puste. Adres URL musi być pełny, to znaczy musi zaczynać się od oznaczenia protokołu czyli od ciągu znaków http:// lub https:// ",
                max_length=1024,
                null=True,
                verbose_name="Dostępna w sieci pod adresem",
            ),
        ),
        migrations.CreateModel(
            name="Obslugujacy_Zgloszenia_Wydzialow",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "wydzial",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="bpp.wydzial"
                    ),
                ),
            ],
            options={
                "verbose_name": "obsługujący zgłoszenia dla wydziału",
                "verbose_name_plural": "obsługujący zgłoszenia dla wydziałów",
                "ordering": ("user__username", "wydzial__nazwa"),
            },
        ),
    ]



# Generated by Django 3.0.14 on 2021-08-18 20:39

from django.db import migrations


def _create_idx(fld, table):
    #  create  index foo114 on pbn_api_publication using gin (upper("title") gin_trgm_ops);
    format = ""
    if fld.find("::") >= 0:
        fld, format = fld.split("::")
        format = f"::{format}"

    idx_name = f"{table[:15]}_{fld}_idx"
    create = f'CREATE INDEX {idx_name} ON {table} USING GIN (UPPER("{fld}"{format}) gin_trgm_ops);'
    delete = f"DROP INDEX {idx_name}"
    return create, delete


def _crt(table, *flds):
    for elem in flds:
        yield migrations.RunSQL(*_create_idx(elem, table))


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0028_gist_idx_publications"),
    ]

    operations = (
        list(
            _crt(
                "pbn_api_institution",
                "name",
                "addressCity",
                "addressStreet",
                "addressPostalCode",
                "polonUid",
                "mongoId",
            )
        )
        + list(
            _crt(
                "pbn_api_journal",
                "title",
                "websiteLink",
                "issn",
                "eissn",
                "mniswId::text",
                "mongoId",
            )
        )
        + list(
            _crt(
                "pbn_api_publisher",
                "publisherName",
                "mniswId::text",
                "mongoId",
            )
        )
        + list(
            _crt(
                "pbn_api_scientist",
                "lastName",
                "name",
                "orcid",
                "mongoId",
            )
        )
    )



# -*- encoding: utf-8 -*-

from django.db import models

# Ta klasa na obecną chwilę nie zawiera nic, ale używamy jej, aby oznaczyć
# odpowiednio pola, gdzie chodzi o rok:
YearField = models.IntegerField

class DOIField(models.CharField):

    def __init__(self, *args, **kw):
        if 'help_text' not in kw:
            kw['help_text'] = "Digital Object Identifier (DOI)"

        if 'max_length' not in kw:
            kw['max_length'] = 2048

        super(DOIField, self).__init__(*args, **kw)



from pbn_api.admin.base import BaseMongoDBAdmin
from pbn_api.admin.filters import MaMNISWIDFilter, OdpowiednikZrodlaWBPPFilter
from pbn_api.models import Journal

from django.contrib import admin


@admin.register(Journal)
class JournalAdmin(BaseMongoDBAdmin):
    list_filter = [
        OdpowiednikZrodlaWBPPFilter,
        MaMNISWIDFilter,
    ] + BaseMongoDBAdmin.list_filter
    list_display = [
        "title",
        "issn",
        "eissn",
        "mniswId",
        "websiteLink",
        "rekord_w_bpp",
    ]
    search_fields = ["mongoId", "title", "websiteLink", "issn", "eissn", "mniswId"]



import pytest

from bpp.util import knapsack, strip_html, wytnij_isbn_z_uwag


@pytest.mark.parametrize(
    "input,output,rest",
    [
        ("ISBN 978-83-7374-091-4, ten tego", "978-83-7374-091-4", "ten tego"),
        ("ISBN 83-200-1817-X", "83-200-1817-X", ""),
        ("tu nie ma ISBNu", None, None),
        ("ISBN-10 978-83-7374-091-4; ISBN-13 958498498894, ten tego", None, None),
    ],
)
def test_wytnij_isbn_z_uwag(input, output, rest):
    res = wytnij_isbn_z_uwag(input)
    if output is None:
        assert res is None
        return

    isbn, reszta = res
    assert isbn == output
    assert reszta == rest


def test_knapsack_empty():
    assert knapsack(10, [], [], []) == (0, [])


@pytest.mark.timeout(3)
def test_knapsack_very_long():
    # Test na optymalizacje sytuacji gdy chcemy dostac znacznie wiecej punktow niz wychdozi z danej listy,
    knapsack(99900, [1] * 99800, [1] * 99800, [1] * 99800)


def test_knapsack_zwracaj_liste():
    assert (
        knapsack(
            10,
            [1, 2, 3, 4, 5],
            [1, 2, 3, 4, 5],
            [
                "a",
                "b",
                "c",
                "d",
                "e",
            ],
            True,
        )
        == (10, ["d", "c", "b", "a"])
    )


def test_knapsack_nie_zwracaj_listy():
    assert (
        knapsack(
            10,
            [1, 2, 3, 4, 5],
            [1, 2, 3, 4, 5],
            [
                "a",
                "b",
                "c",
                "d",
                "e",
            ],
            zwracaj_liste_przedmiotow=False,
        )
        == (10, [])
    )


@pytest.mark.django_db
def test_ModelZOpisemBibliograficznym(wydawnictwo_ciagle):
    assert wydawnictwo_ciagle.opis_bibliograficzny() != ""


@pytest.mark.parametrize(
    "i,o", [("<b>test", "test"), ("<b>test</b>", "test"), ("", ""), (None, None)]
)
def test_strip_html(i, o):
    assert strip_html(i) == o
